@@ -18,29 +18,17 @@ public class CustomElementImpl extends com.umlet.custom.CustomElement {
 		/****CUSTOM_CODE START****/
 //This is a self resizing custom element
 
-//Disable manual resizing
+//Disable manual resizing and enable autoresizing
 allowResize(false);
-
-height=40; //minimal height
-width=10; //minimal width
-//Calculate the width and height of the element
-for(String textline : textlines) {
-	height = height + textheight();
-	width = max(textwidth(textline)+5,width); 
-}
+setAutoresize(30,40,10);
 
 //Draw the circle and the line
 drawCircle(width/2,15,10);
 
-//set the element Location to centered
-//the element will resize to left and right
-//not only to right which is default
-setElementCentered();
-
 //Print out the text
 int y=30;
 for(String textline : textlines) {
-	y = y + textheight();
+	y = y + textHeight();
 	printCenter(textline,y);
 }
 		/****CUSTOM_CODE END****/
