@@ -19,38 +19,29 @@ public class <!CLASSNAME!> extends com.umlet.custom.CustomElement {
 //In addition you are able to resize at manually
 //As soon as it is resized manually a new text is
 //added to the property panel
+setAutoresize(20,20,10);
 
-if(!isManualResized()) {
-	height=20; //minimal height
-	width=20; //minimal width
-	//calculates the width and height of the component
-	for(String textline : textlines) {
-		height = height + textheight();
-		width = max(textwidth(textline)+5,width);
-	}
-}
-
-int y=0;
+int y=textHeight();
 //draws the text
 
 boolean center = true;
 for(int i = 0; i < textlines.size(); i++) {
 	String textline = textlines.get(i);
 	if(textline.equals("--")) {
-		y += 10;
-		height -= (int) (textheight()*0.45);
-		drawLineHorizontal(y-2);
-	center = false;
-   }
-   else {
-		y = y + textheight();
+		drawLineHorizontal((int) (y-textHeight()*0.7));
+		center = false;
+		if (!isManualResized()) height -= (int) (textHeight()*0.75);
+		y += textHeight()*0.3;
+	}
+	else {
 		if (center) printCenter(textline,y); 
-	else printLeft(textline,y); 
-   }
+		else printLeft(textline,y);
+		y += textHeight();
+	}
 }
 
 //draws the outer Rectangle
-drawRect(0,0,onGrid(width),onGrid(height));
+drawRectangle(0,0,onGrid(width),onGrid(height));
 		/****CUSTOM_CODE END****/
 	}
 }
\ No newline at end of file
