@@ -20,18 +20,18 @@ public class <!CLASSNAME!> extends com.umlet.custom.CustomElement {
 //define the element's behavior.
 //
 //Example:  Change the line
-//  y = y + textheight();
+//  y = y + textHeight();
 //to
-//  y = y + 2 * textheight();
+//  y = y + 2 * textHeight();
 //and observe the element preview.
 
-int y=0;
+int y=textHeight();
 
-drawRect(0,0,width,height);
+drawRectangle(0,0,width,height);
 
 for(String textline : textlines) {
-	y = y + textheight();
 	printCenter(textline,y);
+	y = y + textHeight();
 }
 		/****CUSTOM_CODE END****/
 	}
