@@ -22,7 +22,7 @@ public class AddGroup extends Command {
 		super.execute(handler);
 		if (this._entities == null) {
 			this._entities = new Vector<Entity>();
-			this._entities.addAll(handler.getSelectedEntities());
+			this._entities.addAll(handler.getDrawPanel().getSelector().getSelectedEntities());
 		}
 		_group.assignToDiagram(handler);
 		_group.group(this._entities);
