@@ -22,7 +22,7 @@ public class ChangeBGColor extends Command {
 	public void execute(DiagramHandler handler) {
 		super.execute(handler);
 		if (entities == null) {
-			Vector<Entity> es = handler.getSelectedEntities();
+			Vector<Entity> es = handler.getDrawPanel().getSelector().getSelectedEntities();
 			this.entities = new HashMap<Entity, String>();
 			for (Entity e : es)
 				this.entities.put(e, e.getBGColorString());
