@@ -18,22 +18,15 @@ public class Copy extends Command {
 	@Override
 	public void execute(DiagramHandler handler) {
 
-		// We must zoom to the defaultGridsize before execution
-		int oldZoom = handler.getGridSize();
-		handler.setGridAndZoom(Constants.DEFAULTGRIDSIZE, false);
-
 		// dont execute super.execute() because no change of diagram is required.
 		if (this.entities == null) {
 			this.entities = new Vector<Entity>();
-			for (Entity e : handler.getSelectedEntities())
+			for (Entity e : handler.getDrawPanel().getSelector().getSelectedEntities())
 				this.entities.add(e.CloneFromMe());
 		}
-
-		if (this.entities != null) UmletClipBoard.getInstance().copy(this.entities, handler);
-
-		// And zoom back to the oldGridsize after execution
-		handler.setGridAndZoom(oldZoom, false);
-
+		
+//		if (entities.isEmpty()) return; UNCOMMENTED TO ALLOW COPY FULL DIAGRAM TO CLIPBOARD WITHOUT SELECTING ANYTHING
+		UmletClipBoard.getInstance().copy(this.entities, handler);
 	}
 
 	@Override
