@@ -26,42 +26,41 @@ import com.umlet.element.base.Relation;
  */
 
 public class InsertLinePoint extends Command {
-	private Relation _relation;
 
+	private Relation _relation;
+	private int _where;
+
+	private int _x;
+	private int _y;	
+	
 	public Relation getRelation() {
 		return _relation;
 	}
-
-	private int _where;
-
+	
 	public int getWhere() {
 		return _where;
 	}
 
-	private int _x;
-
-	public int getX() {
-		return _x;
+	private int getX() {
+		return _x * _relation.getHandler().getGridSize();
 	}
 
-	private int _y;
-
-	public int getY() {
-		return _y;
+	private int getY() {
+		return _y * _relation.getHandler().getGridSize();
 	}
 
 	public InsertLinePoint(Relation r, int i, int x, int y) {
 		_relation = r;
 		_where = i;
-		_x = x;
-		_y = y;
+		_x = x / _relation.getHandler().getGridSize();
+		_y = y / _relation.getHandler().getGridSize();
 	}
 
 	@Override
 	public void execute(DiagramHandler handler) {
 		super.execute(handler);
 		Vector<Point> tmp = _relation.getLinePoints();
-		tmp.insertElementAt(new Point(_x, _y), _where);
+		tmp.insertElementAt(new Point(getX(), getY()), _where);
 		_relation.repaint();
 	}
 
