@@ -26,36 +26,36 @@ import com.umlet.element.base.Relation;
  */
 
 public class RemoveLinePoint extends Command {
+
 	private Relation _relation;
+	private int _where;
+
+	private int _x;
+	private int _y;
 
 	public Relation getRelation() {
 		return _relation;
-	}
-
-	private int _where;
-
+	}	
+	
 	public int getWhere() {
 		return _where;
+	}	
+	
+	private int getX() {
+		return _x * _relation.getHandler().getGridSize();
 	}
 
-	private int _x;
-
-	public int getX() {
-		return _x;
-	}
-
-	private int _y;
-
-	public int getY() {
-		return _y;
+	private int getY() {
+		return _y * _relation.getHandler().getGridSize();
 	}
 
 	public RemoveLinePoint(Relation r, int i) {
 		_relation = r;
 		_where = i;
 		Point p = r.getLinePoints().elementAt(i);
-		_x = p.x;
-		_y = p.y;
+		
+		_x = p.x / _relation.getHandler().getGridSize();
+		_y = p.y / _relation.getHandler().getGridSize();		
 	}
 
 	@Override
@@ -70,7 +70,7 @@ public class RemoveLinePoint extends Command {
 	public void undo(DiagramHandler handler) {
 		super.undo(handler);
 		Vector<Point> tmp = _relation.getLinePoints();
-		tmp.insertElementAt(new Point(_x, _y), _where);
+		tmp.insertElementAt(new Point(getX(), getY()), _where);
 		_relation.repaint();
 	}
 }
