@@ -4,32 +4,41 @@ import java.awt.Dimension;
 import java.awt.Font;
 import java.awt.Graphics2D;
 import java.awt.Point;
-import java.awt.font.FontRenderContext;
 import java.awt.font.TextLayout;
 import java.awt.geom.Rectangle2D;
 import java.awt.print.PrinterException;
 import java.awt.print.PrinterJob;
 import java.io.File;
+import java.io.IOException;
+import java.util.Timer;
+import java.util.TimerTask;
 import java.util.Vector;
 
 import javax.swing.JOptionPane;
 
+import org.apache.log4j.Logger;
+
 import com.umlet.constants.Constants;
 import com.umlet.control.Umlet;
 import com.umlet.control.io.DiagramFileHandler;
 import com.umlet.element.base.Entity;
 import com.umlet.element.base.Group;
 import com.umlet.element.base.Relation;
+import com.umlet.gui.base.DiagramPopupMenu;
 import com.umlet.gui.base.StartUpHelpText;
+import com.umlet.gui.base.UmletGUI;
 import com.umlet.gui.base.listeners.DiagramListener;
 import com.umlet.gui.base.listeners.EntityListener;
 import com.umlet.gui.base.listeners.RelationListener;
 import com.umlet.gui.standalone.StandaloneGUI;
 
 public class DiagramHandler {
+
+	private static Logger log = Logger.getLogger(DiagramHandler.class);
+
 	private boolean isChanged;
 	private DiagramFileHandler fileHandler;
-	private DrawPanel drawpanel;
+	protected DrawPanel drawpanel;
 	private Controller controller;
 	protected DiagramListener listener;
 	private String helptext;
@@ -49,12 +58,27 @@ public class DiagramHandler {
 		this.controller = new Controller(this);
 		this.fileHandler = DiagramFileHandler.createInstance(this, diagram);
 		if (!nolistener) this.setListener(new DiagramListener(this));
-		new StartUpHelpText(drawpanel); // Create a new startup help panel
 		if (diagram != null) this.fileHandler.doOpen();
 		if (this.helptext == null) this.helptext = Constants.DEFAULT_HELPTEXT;
+
+		boolean extendedPopupMenu = false;
 		if (Umlet.getInstance().getGUI() instanceof StandaloneGUI) {
 			((StandaloneGUI) Umlet.getInstance().getGUI()).setValueOfZoomDisplay(getGridSize());
+			extendedPopupMenu = true; // AB: use extended popup menu on standalone gui only
 		}
+
+		this.drawpanel.setComponentPopupMenu(new DiagramPopupMenu(this, extendedPopupMenu));
+
+		// Wait until drawpanel is valid and then update panel and scrollbars
+		new Timer().schedule(new TimerTask() {
+			@Override
+			public void run() {
+				if (drawpanel.isValid()) {
+					drawpanel.updatePanelAndScrollbars();
+					cancel();
+				}
+			}
+		}, 25, 25);
 	}
 
 	public void setEnabled(boolean en) {
@@ -120,14 +144,14 @@ public class DiagramHandler {
 	public Dimension getZoomedTextSize(Graphics2D g2, String s) {
 		if (s == null) return null;
 		if (s.length() == 0) return new Dimension(0, 0);
-		TextLayout tl = new TextLayout(s, getZoomedFont(), getFRC(g2));
+		TextLayout tl = new TextLayout(s, getZoomedFont(), Constants.getFRC(g2));
 		return new Dimension((int) tl.getBounds().getWidth(), (int) tl.getBounds().getHeight());
 	}
 
 	public Dimension getRealTextSize(Graphics2D g2, String s) {
 		if (s == null) return null;
 		if (s.length() == 0) return new Dimension(0, 0);
-		TextLayout tl = new TextLayout(s, getRealFont(), getFRC(g2));
+		TextLayout tl = new TextLayout(s, getRealFont(), Constants.getFRC(g2));
 		return new Dimension((int) tl.getBounds().getWidth(), (int) tl.getBounds().getHeight());
 	}
 
@@ -167,13 +191,6 @@ public class DiagramHandler {
 		return new Font(f.getName(), Font.BOLD | Font.ITALIC, f.getSize());
 	}
 
-	public FontRenderContext getFRC(Graphics2D g2) {
-		FontRenderContext rendering;
-		rendering = new FontRenderContext(null, true, true);
-		g2.setRenderingHints(Constants.getUxRenderingQualityHigh());
-		return rendering;
-	}
-
 	public void write(Graphics2D g2, String s, int x, int y, int align, int valign) {
 		if (s == null) return;
 
@@ -203,7 +220,7 @@ public class DiagramHandler {
 
 		g2.drawString(s, x, y);
 		if (underline) {
-			TextLayout l = new TextLayout(s, g2.getFont(), getFRC(g2));
+			TextLayout l = new TextLayout(s, g2.getFont(), Constants.getFRC(g2));
 			Rectangle2D r2d = l.getBounds();
 			if (bold) {
 				g2.drawLine(x, y + (int) getZoomedDistTextToLine() / 2, x + (int) r2d.getWidth(), y + (int) getZoomedDistTextToLine() / 2);
@@ -301,7 +318,8 @@ public class DiagramHandler {
 	public void setChanged(boolean changed) {
 		if (this.isChanged != changed) {
 			this.isChanged = changed;
-			Umlet.getInstance().getGUI().setDiagramChanged(this, changed);
+			UmletGUI gui = Umlet.getInstance().getGUI();
+			if (gui != null) gui.setDiagramChanged(this, changed);
 		}
 	}
 
@@ -309,10 +327,6 @@ public class DiagramHandler {
 		return this.drawpanel;
 	}
 
-	public Vector<Entity> getSelectedEntities() {
-		return this.drawpanel.getSelector().getSelectedEntities();
-	}
-
 	public DiagramFileHandler getFileHandler() {
 		return this.fileHandler;
 	}
@@ -324,18 +338,25 @@ public class DiagramHandler {
 	// returnvalue needed for eclipse plugin
 	// returns true if the file is saved, else returns false
 	public boolean doSave() {
-		if (this.fileHandler.doSave()) {
+		try {
+			this.fileHandler.doSave();
 			this.setChanged(false);
 			this.reloadPalettes();
 			return true;
+		} catch (IOException e) {
+			Umlet.displayError(Constants.ERROR_SAVING_FILE);
+			return false;
 		}
-		else return false;
 	}
 
 	public void doSaveAs(String extension) {
-		if (this.fileHandler.doSaveAs(extension)) {
+		if (this.drawpanel.getAllEntities().isEmpty()) Umlet.displayError(Constants.ERROR_SAVING_EMPTY_DIAGRAM);
+		try {
+			this.fileHandler.doSaveAs(extension);
 			this.setChanged(false);
 			this.reloadPalettes();
+		} catch (IOException e) {
+			Umlet.displayError(Constants.ERROR_SAVING_FILE);
 		}
 	}
 
@@ -349,7 +370,7 @@ public class DiagramHandler {
 		if (printJob.printDialog()) try {
 			printJob.print();
 		} catch (PrinterException pe) {
-			Umlet.displayError("An error occured during printing.");
+			Umlet.displayError(Constants.ERROR_PRINTING);
 		}
 	}
 
@@ -441,122 +462,163 @@ public class DiagramHandler {
 		this.gridSize = gridSize;
 	}
 
-	public int ensureOnGrid(int val) {
-		return ensureOnGrid(val, false);
+	public int realignToGrid(int val) {
+		return realignToGrid(true, val, false);
+	}
+
+	public int realignToGrid(boolean logRealign, int val) {
+		return realignToGrid(logRealign, val, false);
 	}
 
 	/**
 	 * returns the integer which is nearest to val but on the grid (round down)
-	 * @param val value which should be rounded to the grid
+	 * 
+	 * @param logRealign
+	 *            if true a realign is logged as an error
+	 * @param val
+	 *            value which should be rounded to the grid
+	 * @param roundUp
+	 *            if true the realign rounds up instead of down
 	 * @return value on the grid
 	 */
-	public int ensureOnGrid(int val, boolean roundUp) {
-		if (Math.abs(val) % gridSize != 0) {
+	public int realignToGrid(boolean logRealign, int val, boolean roundUp) {
+		if (val % gridSize != 0) {
+			if (logRealign) log.error("realignToGrid");
 			val -= val % gridSize;
 			if (roundUp) val += gridSize;
 		}
 		return val;
 	}
 
-	public void setGridAndZoom(int factor) {
-		setGridAndZoom(factor, true);
+	static public int realignTo(int val, int toVal) {
+		if (val % toVal != 0) {
+			val -= val % toVal;
+		}
+		return val;
 	}
 
-	public void setGridAndZoom(int factor, boolean zoomToTheCenter) {
+	static public void zoomEntity(int fromFactor, int toFactor, Entity e) {
+		Vector<Entity> vec = new Vector<Entity>();
+		vec.add(e);
+		zoomEntities(fromFactor,toFactor,vec);
+	}
+
+	static public void zoomEntities(int fromFactor, int toFactor, Vector<Entity> selectedEntities) {
 
 		/**
-		 * Store the old gridsize and the new one. Furthermore check if the zoom process must be made
+		 * The entities must be resized to the new factor
 		 */
 
-		int oldGridSize = getGridSize();
+		for (Entity entity : selectedEntities) {
+			// Entities in groups are not part of the selectedEntities vector. therefore they must be zoomed explicitely
+			if (entity instanceof Group) zoomEntities(fromFactor, toFactor, ((Group) entity).getMembers());
 
-		if ((factor < 1) || (factor > 20)) return; // Only zoom between 10% and 200% is allowed
-		if (factor == oldGridSize) return; // Only zoom is gridsize has changed
+			int newX = (entity.getX() * toFactor) / fromFactor;
+			int newY = (entity.getY() * toFactor) / fromFactor;
+			int newW = (entity.getWidth() * toFactor) / fromFactor;
+			int newH = (entity.getHeight() * toFactor) / fromFactor;
+			entity.setLocation(realignTo(newX, toFactor), realignTo(newY, toFactor));
+			// Normally there should be no realign here but relations and custom elements sometimes must be realigned therefore we don't log it as an error
+			entity.setSize(realignTo(newW, toFactor), realignTo(newH, toFactor));
 
-		setGridSize(factor);
-
-		/**
-		 * The entities must be resized to the new gridsize
-		 */
-
-		// We must resize all entitys to match the new zoom level
-		for (Entity entity : getDrawPanel().getAllEntities()) {
-
-			// Groups are handled separately later
-			if (entity instanceof Group) continue;
-
-			int newX = (entity.getX() * gridSize) / oldGridSize;
-			int newY = (entity.getY() * gridSize) / oldGridSize;
-			int newW = (entity.getWidth() * gridSize) / oldGridSize;
-			int newH = (entity.getHeight() * gridSize) / oldGridSize;
-			entity.setLocation(ensureOnGrid(newX), ensureOnGrid(newY));
-			entity.setSize(ensureOnGrid(newW), ensureOnGrid(newH));
-
-			// We must also resize the coordinates of the points of the relations
+			// Resize the coordinates of the points of the relations
 			if (entity instanceof Relation) {
 				for (Point point : ((Relation) entity).getLinePoints()) {
-					newX = ((int) point.getX() * gridSize) / oldGridSize;
-					newY = ((int) point.getY() * gridSize) / oldGridSize;
-					point.setLocation(ensureOnGrid(newX), ensureOnGrid(newY));
+					newX = ((int) point.getX() * toFactor) / fromFactor;
+					newY = ((int) point.getY() * toFactor) / fromFactor;
+					point.setLocation(realignTo(newX, toFactor), realignTo(newY, toFactor));
 				}
 			}
 		}
 
-		/**
-		 * The zoomed diagram will shrink to the upper left corner and grow to the lower right
-		 * corner but we want to have the zoom center in the middle of the actual visible drawpanel
-		 * so we have to change the coordinates of the entities
-		 */
-
-		if (zoomToTheCenter) {
-			// We calculate the middle point of the actual diagramspanel size
-			int x = (int) (getDrawPanel().getViewableDiagrampanelSize().getWidth() / 2);
-			int y = (int) (getDrawPanel().getViewableDiagrampanelSize().getHeight() / 2);
-			// And add any space on the upper left corner which is not visible but reachable by scrollbar
-			x += getDrawPanel().getScrollPanel().getViewport().getViewPosition().getX();
-			y += getDrawPanel().getScrollPanel().getViewport().getViewPosition().getY();
-
-			// The result is the point where we want to center the zoom of the diagram
-			int diffx, diffy;
-			diffx = x - x * gridSize / oldGridSize;
-			diffy = y - y * gridSize / oldGridSize;
-
-			for (Entity e : getDrawPanel().getAllEntities()) {
-				// Groups are handled separately later
-				if (!(e instanceof Group)) e.changeLocation(ensureOnGrid(diffx), ensureOnGrid(diffy));
-			}
-		}
-
 		/**
 		 * After all location and size changes we must adjust the size of the
 		 * top level groups (groups who aren't part of other groups)
 		 */
 
-		for (Entity entity : getDrawPanel().getAllEntities()) {
+		for (Entity entity : selectedEntities) {
 			if ((entity instanceof Group) && !entity.isPartOfGroup()) {
 				((Group) entity).adjustSize(true);
 			}
 		}
 
-		/**
-		 * Now we have to do some additional "clean up" stuff which is related to the zoom
-		 */
-
-		// We realign everything to the grid to be compatible to old diagrams where not everything was on grid
-		getDrawPanel().warnIfAnyValueIsNotOnGrid();
-
-		// We must adjust the coordinates for the scrollbars and revalidate the drawpanel
-		getDrawPanel().insertUpperLeftWhitespaceIfNeeded();
-		getDrawPanel().removeUnnecessaryWhitespaceAroundDiagram();
-		getDrawPanel().revalidate();
-
-		// We set the isChanged variable to allow saving in EclipsePlugin and to reflect the changes made by zooming
-		setChanged(true);
-
-		// If this is the standalone client we must also set the zoom value on the upper panel
-		if (Umlet.getInstance().getGUI() instanceof StandaloneGUI) {
-			((StandaloneGUI) Umlet.getInstance().getGUI()).setValueOfZoomDisplay(factor);
-		}
 	}
 
+	public void setGridAndZoom(int factor) {
+		setGridAndZoom(factor, true);
+	}
+
+	public void setGridAndZoom(int factor, boolean manualZoom) {
+
+		/**
+		 * Store the old gridsize and the new one. Furthermore check if the zoom process must be made
+		 */
+
+		int oldGridSize = getGridSize();
+
+		if ((factor < 1) || (factor > 20)) return; // Only zoom between 10% and 200% is allowed
+		if (factor == oldGridSize) return; // Only zoom if gridsize has changed
+
+		setGridSize(factor);
+
+		/**
+		 * Zoom entities to the new gridsize
+		 */
+
+		zoomEntities(oldGridSize, gridSize, getDrawPanel().getAllEntitiesNotInGroup());
+
+		// AB: Zoom origin
+		getDrawPanel().zoomOrigin(oldGridSize, gridSize);
+
+		/**
+		 * The zoomed diagram will shrink to the upper left corner and grow to the lower right
+		 * corner but we want to have the zoom center in the middle of the actual visible drawpanel
+		 * so we have to change the coordinates of the entities again
+		 */
+
+		if (manualZoom) {
+			// Calculate the middle point of the actual diagramspanel size
+			float x = ((float) getDrawPanel().getViewableDiagrampanelSize().getWidth() / 2);
+			float y = ((float) getDrawPanel().getViewableDiagrampanelSize().getHeight() / 2);
+
+			// And add any space on the upper left corner which is not visible but reachable by scrollbar
+			x += getDrawPanel().getScrollPanel().getViewport().getViewPosition().getX();
+			y += getDrawPanel().getScrollPanel().getViewport().getViewPosition().getY();
+
+			// The result is the point where we want to center the zoom of the diagram
+			float diffx, diffy;
+			diffx = x - (x * gridSize / oldGridSize);
+			diffy = y - (y * gridSize / oldGridSize);
+
+			// AB: Move origin in opposite direction
+			log.debug("diffX/diffY: " + diffx + "/" + diffy);
+			log.debug("Manual Zoom Delta: " + realignToGrid(false, (int) diffx) + "/" + realignToGrid(false, (int) diffy));
+			getDrawPanel().moveOrigin(realignToGrid(false, -(int) diffx), realignToGrid(false, -(int) diffy));
+
+			for (Entity e : getDrawPanel().getAllEntitiesNotInGroup()) {
+				e.changeLocation(realignToGrid(false, (int) diffx), realignToGrid(false, (int) diffy));
+			}
+
+			/**
+			 * Now we have to do some additional "clean up" stuff which is related to the zoom
+			 */
+
+			getDrawPanel().updatePanelAndScrollbars();
+
+			// Set the isChanged variable to allow saving in EclipsePlugin and to reflect the changes made by zooming
+			setChanged(true);
+
+			// If this is the standalone client the zoom valie on the upper panel must also be set
+			UmletGUI gui = Umlet.getInstance().getGUI();
+			if ((gui != null) && (gui instanceof StandaloneGUI)) {
+				((StandaloneGUI) Umlet.getInstance().getGUI()).setValueOfZoomDisplay(factor);
+			}
+
+			float zoomFactor = Umlet.getInstance().getDiagramHandler().getZoomFactor() * 100;
+			String zoomtext;
+			if (Umlet.getInstance().getDiagramHandler() instanceof PaletteHandler) zoomtext = "Palette zoomed to " + (new Integer((int) zoomFactor).toString()) + "%";
+			else zoomtext = "Diagram zoomed to " + (new Integer((int) zoomFactor).toString()) + "%";
+			Umlet.getInstance().getCurrentInfoDiagram().getDrawPanel().showNotification(zoomtext);
+		}
+	}
 }
