@@ -10,30 +10,45 @@ import java.awt.Rectangle;
 import java.awt.geom.AffineTransform;
 import java.awt.print.PageFormat;
 import java.awt.print.Printable;
+import java.util.ArrayList;
+import java.util.List;
 import java.util.Vector;
 
-import javax.swing.JOptionPane;
 import javax.swing.JPanel;
 import javax.swing.JScrollPane;
 import javax.swing.RepaintManager;
 import javax.swing.ScrollPaneConstants;
 
+import org.apache.log4j.Logger;
+
 import com.umlet.constants.Constants;
+import com.umlet.element.base.DiagramNotification;
 import com.umlet.element.base.Entity;
 import com.umlet.element.base.Relation;
+import com.umlet.gui.base.StartUpHelpText;
 import com.umlet.gui.base.listeners.ScrollbarListener;
 
 @SuppressWarnings("serial")
 public class DrawPanel extends JPanel implements Printable {
+
+	private static Logger log = Logger.getLogger(DrawPanel.class);
+
+	private Point origin;
 	private JScrollPane _scr;
 	private Selector selector;
 	private DiagramHandler handler;
+	private List<DiagramNotification> notifications;
 
 	public DrawPanel(DiagramHandler handler) {
 		this.handler = handler;
+		// AB: Origin is used to track diagram movement in Cut Command
+		this.origin = new Point();
 		this.setLayout(null);
 		this.setBackground(Color.white);
+		// If this is not a palette, create a StartupHelpText
+		if (!(handler instanceof PaletteHandler)) this.add(new StartUpHelpText(this));
 		this.selector = new Selector(this);
+		this.notifications = new ArrayList<DiagramNotification>();
 		JScrollPane p = new JScrollPane() {
 			@Override
 			public void setEnabled(boolean en) {
@@ -41,10 +56,13 @@ public class DrawPanel extends JPanel implements Printable {
 				this.getViewport().getView().setEnabled(en);
 			}
 		};
-		p.getHorizontalScrollBar().setUnitIncrement(20);
+
+		p.getHorizontalScrollBar().setUnitIncrement(Constants.DEFAULTGRIDSIZE);
+		p.getHorizontalScrollBar().setBlockIncrement(Constants.DEFAULTGRIDSIZE);
 		p.getHorizontalScrollBar().setSize(0, 15);
 		p.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
-		p.getVerticalScrollBar().setUnitIncrement(20);
+		p.getVerticalScrollBar().setUnitIncrement(Constants.DEFAULTGRIDSIZE);
+		p.getVerticalScrollBar().setBlockIncrement(Constants.DEFAULTGRIDSIZE);
 		p.getVerticalScrollBar().setSize(15, 0);
 		p.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
 
@@ -54,7 +72,7 @@ public class DrawPanel extends JPanel implements Printable {
 
 		p.setBorder(null);
 		this.setScrollPanel(p);
-
+		this.repaint(); //repaint the drawpanel to be sure everything is visible (startuphelp etc)
 	}
 
 	@Override
@@ -82,17 +100,20 @@ public class DrawPanel extends JPanel implements Printable {
 
 	/**
 	 * Returns the smalles possible rectangle which contains all entities and a border space around it
+	 * 
+	 * @param borderSpace the borderspace around the rectangle
+	 * @param entities the entities which should be included
+	 * @return
 	 */
-	public Rectangle getContentBounds(int borderSpace) {
+	public Rectangle getContentBounds(int borderSpace, Vector<Entity> entities) {
 
 		int minx = Integer.MAX_VALUE;
 		int miny = Integer.MAX_VALUE;
 		int maxx = 0;
 		int maxy = 0;
 
-		Vector<Entity> v = getAllEntities();
-		for (int i = 0; i < v.size(); i++) {
-			Entity e = v.elementAt(i);
+		for (int i = 0; i < entities.size(); i++) {
+			Entity e = entities.elementAt(i);
 			minx = Math.min(minx, e.getX() - borderSpace);
 			miny = Math.min(miny, e.getY() - borderSpace);
 			maxx = Math.max(maxx, e.getX() + e.getWidth() + borderSpace);
@@ -101,13 +122,19 @@ public class DrawPanel extends JPanel implements Printable {
 		return new Rectangle(minx, miny, maxx - minx, maxy - miny);
 	}
 
-	public void paintEntitiesIntoGraphics2D(Graphics2D g2d) {
-		Vector<Entity> allEntities = getAllEntities();
-		for (Entity entity : allEntities) {
+	public void paintEntitiesIntoGraphics2D(Graphics2D g2d, Vector<Entity> entitiesToPaint) {
+		boolean entityWasSelected = false;
+		for (Entity entity : entitiesToPaint) {
+			if (entity.isSelected()) entityWasSelected = true;
 			int x = entity.getX();
 			int y = entity.getY();
 			g2d.translate(x, y);
+			if (entityWasSelected) entity.onDeselected(); // If entity was selected deselect it before painting
 			entity.paint(g2d);
+			if (entityWasSelected) { // and select it afterwards. Also reset boolean variable to false
+				entity.onSelected();
+				entityWasSelected = false;
+			}
 			g2d.translate(-x, -y);
 		}
 	}
@@ -118,7 +145,7 @@ public class DrawPanel extends JPanel implements Printable {
 			Graphics2D g2d = (Graphics2D) g;
 			RepaintManager currentManager = RepaintManager.currentManager(this);
 			currentManager.setDoubleBufferingEnabled(false);
-			Rectangle bounds = this.getContentBounds(Constants.PRINTPADDING);
+			Rectangle bounds = this.getContentBounds(Constants.PRINTPADDING, getAllEntities());
 			g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
 			AffineTransform t = g2d.getTransform();
 			double scale = Math.min(pageFormat.getImageableWidth() / bounds.width,
@@ -144,16 +171,15 @@ public class DrawPanel extends JPanel implements Printable {
 		return v;
 	}
 
-	public Vector<Entity> getNotInGroupEntitiesOnPanel() {
-		Vector<Entity> all = this.getAllEntities();
+	public Vector<Entity> getAllEntitiesNotInGroup() {
 		Vector<Entity> entities = new Vector<Entity>();
-		for (Entity e : all) {
+		for (Entity e : getAllEntities()) {
 			if (!e.isPartOfGroup()) entities.add(e);
 		}
 		return entities;
 	}
 
-	public Vector<Relation> getAllRelationsOnPanel() {
+	public Vector<Relation> getAllRelations() {
 		Component[] tmp = this.getComponents();
 		Vector<Relation> ret = new Vector<Relation>();
 		for (int i = 0; i < tmp.length; i++) {
@@ -166,49 +192,32 @@ public class DrawPanel extends JPanel implements Printable {
 		return this.selector;
 	}
 
-	public void warnIfAnyValueIsNotOnGrid() {
-		Vector<Entity> v = this.getAllEntities();
-		for (int i = 0; i < v.size(); i++) {
-			Entity entity = v.elementAt(i);
-			int x = entity.getX();
-			int y = entity.getY();
-			int w = entity.getWidth();
-			int h = entity.getHeight();
-
-			if (x != getHandler().ensureOnGrid(x)) JOptionPane.showMessageDialog(this, entity + " is not on Grid -> X: " + x);
-			if (y != getHandler().ensureOnGrid(y)) JOptionPane.showMessageDialog(this, entity + " is not on Grid -> Y: " + y);
-			if (w != getHandler().ensureOnGrid(w)) JOptionPane.showMessageDialog(this, entity + " is not on Grid -> W: " + w);
-			if (h != getHandler().ensureOnGrid(h)) JOptionPane.showMessageDialog(this, entity + " is not on Grid -> H: " + h);
-
-			if (entity instanceof Relation) {
-				for (Point point : ((Relation) entity).getLinePoints()) {
-					x = (int) point.getX();
-					y = (int) point.getY();
-					if (x != getHandler().ensureOnGrid(x)) JOptionPane.showMessageDialog(this, "Point of " + entity + " is not on Grid -> X: " + x);
-					if (y != getHandler().ensureOnGrid(y)) JOptionPane.showMessageDialog(this, "Point of " + entity + " is not on Grid -> Y: " + y);
-				}
-			}
-		}
+	public void updatePanelAndScrollbars() {
+		insertUpperLeftWhitespaceIfNeeded();
+		removeUnnecessaryWhitespaceAroundDiagram();
 	}
 
 	/**
 	 * If entities are out of the visible drawpanel border on the upper left
 	 * corner this method enlarges the drawpanel and displays scrollbars
 	 */
-	public void insertUpperLeftWhitespaceIfNeeded() {
+	private void insertUpperLeftWhitespaceIfNeeded() {
 
-		Rectangle diaWithoutWhite = getContentBounds(0);
+		Rectangle diaWithoutWhite = getContentBounds(0, getAllEntities());
 		// We must adjust the components and the view by a certain factor
 		int adjustWidth = 0;
 		if (diaWithoutWhite.getX() < 0) adjustWidth = (int) diaWithoutWhite.getX();
 
 		int adjustHeight = 0;
 		if (diaWithoutWhite.getY() < 0) adjustHeight = (int) diaWithoutWhite.getY();
+
+		moveOrigin(adjustWidth, adjustHeight);
+
 		// If any adjustment is needed we move the components and increase the view position
 		if ((adjustWidth != 0) || (adjustHeight != 0)) {
 			for (int i = 0; i < this.getComponents().length; i++) {
 				Component c = this.getComponent(i);
-				c.setLocation(handler.ensureOnGrid(c.getX() - adjustWidth), handler.ensureOnGrid(c.getY() - adjustHeight));
+				c.setLocation(handler.realignToGrid(false, c.getX() - adjustWidth), handler.realignToGrid(false, c.getY() - adjustHeight));
 			}
 		}
 
@@ -235,23 +244,34 @@ public class DrawPanel extends JPanel implements Printable {
 	 * If there is a scrollbar visible and a unnecessary whitespace on any border of the diagram
 	 * which is not visible (but possibly scrollable by scrollbars) we remove this whitespace
 	 */
-	public void removeUnnecessaryWhitespaceAroundDiagram() {
+	private void removeUnnecessaryWhitespaceAroundDiagram() {
 
-		Rectangle diaWithoutWhite = getContentBounds(0);
+		Rectangle diaWithoutWhite = getContentBounds(0,getAllEntities());
 		Dimension viewSize = getViewableDiagrampanelSize();
 		int horSbPos = _scr.getHorizontalScrollBar().getValue();
 		int verSbPos = _scr.getVerticalScrollBar().getValue();
 
+		horSbPos = handler.realignToGrid(false, horSbPos);
+		verSbPos = handler.realignToGrid(false, verSbPos);
+
 		int newX = 0;
 		if (_scr.getHorizontalScrollBar().isShowing()) {
-			if (horSbPos > diaWithoutWhite.getX()) newX = (int) diaWithoutWhite.getX();
-			else newX = horSbPos;
+			if (horSbPos > diaWithoutWhite.getX()) {
+				newX = (int) diaWithoutWhite.getX();
+			}
+			else {
+				newX = horSbPos;
+			}
 		}
 
 		int newY = 0;
 		if (_scr.getVerticalScrollBar().isShowing()) {
-			if (verSbPos > diaWithoutWhite.getY()) newY = (int) diaWithoutWhite.getY();
-			else newY = verSbPos;
+			if (verSbPos > diaWithoutWhite.getY()) {
+				newY = (int) diaWithoutWhite.getY();
+			}
+			else {
+				newY = verSbPos;
+			}
 		}
 
 		int newWidth = (int) (horSbPos + viewSize.getWidth());
@@ -266,9 +286,12 @@ public class DrawPanel extends JPanel implements Printable {
 			newHeight = (int) (diaWithoutWhite.getY() + diaWithoutWhite.getHeight());
 		}
 
+		moveOrigin(newX, newY);
+
 		for (int i = 0; i < this.getComponents().length; i++) {
 			Component c = this.getComponent(i);
-			c.setLocation(handler.ensureOnGrid(c.getX() - newX), handler.ensureOnGrid(c.getY() - newY));
+			if (c instanceof Entity) // We remove whitespace only for entities
+			c.setLocation(handler.realignToGrid(false, c.getX() - newX), handler.realignToGrid(false, c.getY() - newY));
 		}
 
 		changeViewPosition(-newX, -newY);
@@ -281,7 +304,7 @@ public class DrawPanel extends JPanel implements Printable {
 	 * Returns the visible size of the drawpanel
 	 */
 	public Dimension getViewableDiagrampanelSize() {
-		return _scr.getViewport().getSize();
+		return getVisibleRect().getSize();
 	}
 
 	private void checkIfScrollbarsAreNecessary() {
@@ -290,17 +313,17 @@ public class DrawPanel extends JPanel implements Printable {
 		 * This is needed to avoid appearing scrollbars if the diagramm is on the bottom right
 		 */
 
-		Rectangle diaWithoutWhite = getContentBounds(0);
+		Rectangle diaWithoutWhite = getContentBounds(0, getAllEntities());
 		Dimension viewSize = getViewableDiagrampanelSize();
 
+		boolean vertWasVisible = isVerticalScrollbarVisible();
+		boolean horWasVisible = isHorizontalScrollbarVisible();
+
 		// Only if the scrollbar is visible we must respect its size to calculate the visibility of the scrollbar
 		int verSbWidth = 0;
 		int horSbHeight = 0;
-		if (_scr.getVerticalScrollBar().isShowing()) verSbWidth = _scr.getVerticalScrollBar().getWidth();
-		if (_scr.getHorizontalScrollBar().isShowing()) horSbHeight = _scr.getHorizontalScrollBar().getHeight();
-
-		boolean vertWasVisible = isVerticalScrollbarVisible();
-		boolean horWasVisible = isHorizontalScrollbarVisible();
+		if (vertWasVisible) verSbWidth = _scr.getVerticalScrollBar().getWidth();
+		if (horWasVisible) horSbHeight = _scr.getHorizontalScrollBar().getHeight();
 
 		// If the horizontal scrollbar is on the most left point && the the right end of the diagram without whitespace <= the viewable width incl. the width of the vertical scrollbar we hide the horizontal scrollbar
 		if ((_scr.getHorizontalScrollBar().getValue() < handler.getGridSize()) && ((diaWithoutWhite.getX() + diaWithoutWhite.getWidth()) <= (viewSize.getWidth() + verSbWidth))) setHorizontalScrollbarVisibility(false);
@@ -312,18 +335,18 @@ public class DrawPanel extends JPanel implements Printable {
 		if ((_scr.getVerticalScrollBar().getValue() < handler.getGridSize()) && ((diaWithoutWhite.getY() + diaWithoutWhite.getHeight()) <= (viewSize.getHeight() + horSbHeight))) setVerticalScrollbarVisibility(false);
 		else if ((_scr.getVerticalScrollBar().getValue() < handler.getGridSize()) && (getViewableDiagrampanelSize().getHeight() + _scr.getVerticalScrollBar().getValue() == diaWithoutWhite.getY() + diaWithoutWhite.getHeight())) setVerticalScrollbarVisibility(false);
 		else setVerticalScrollbarVisibility(true);
+
+		// REMOVED TO FIX JUMPING PALETTE ENTRIES AT COPYING/CUTTING
 		// adjust x and y to avoid jumping diagram if both scrollbars were visible and one of them disappears (only in the upper left corner)
 		int adx = 0;
 		int ady = 0;
-		if ((_scr.getHorizontalScrollBar().getValue() != 0) && vertWasVisible && !isVerticalScrollbarVisible()) adx = handler.ensureOnGrid(horSbHeight);
-		if ((_scr.getVerticalScrollBar().getValue() != 0) && horWasVisible && !isHorizontalScrollbarVisible()) ady = handler.ensureOnGrid(verSbWidth);
+		if ((_scr.getHorizontalScrollBar().getValue() != 0) && vertWasVisible && !isVerticalScrollbarVisible()) adx = handler.realignToGrid(false, horSbHeight);
+		if ((_scr.getVerticalScrollBar().getValue() != 0) && horWasVisible && !isHorizontalScrollbarVisible()) ady = handler.realignToGrid(false, verSbWidth);
 
 		if ((adx != 0) || (ady != 0)) {
 			setPreferredSize(new Dimension((int) (getPreferredSize().getWidth() + adx), (int) getPreferredSize().getHeight() + ady));
 			changeViewPosition(adx, ady);
 		}
-
-		revalidate();
 	}
 
 	private void setHorizontalScrollbarVisibility(boolean visible) {
@@ -362,9 +385,27 @@ public class DrawPanel extends JPanel implements Printable {
 		}
 	}
 
+	private void drawDevHelpLines(Graphics2D g2d) {
+		g2d.setStroke(Constants.getStroke(1, 1));
+
+		g2d.setColor(Color.BLUE);
+		int w = handler.getDrawPanel().getScrollPanel().getViewport().getViewPosition().x;
+		int h = handler.getDrawPanel().getScrollPanel().getViewport().getViewPosition().y;
+		g2d.drawRect(w, h, w + 2, h + 2);
+
+		g2d.setColor(Color.GRAY);
+		Dimension dim = getViewableDiagrampanelSize();
+		g2d.drawRect(0, 0, (int) dim.getWidth(), (int) dim.getHeight());
+
+		g2d.setColor(Color.RED);
+		Dimension dim2 = getPreferredSize();
+		g2d.drawRect(0, 0, (int) dim2.getWidth(), (int) dim2.getHeight());
+
+		g2d.setStroke(Constants.getStroke(0, 1));
+	}
+
 	@Override
 	protected void paintChildren(Graphics g) {
-
 		// Possible Zoom of the drawpanel (not a real zoom because it doesn't resize anything)
 		// Graphics2D g2 = (Graphics2D)g;
 		// AffineTransform t = g2.getTransform();
@@ -372,28 +413,81 @@ public class DrawPanel extends JPanel implements Printable {
 		// t.scale(scale, scale);
 		// g2.setTransform(t);
 
+		// drawDevHelpLines((Graphics2D) g);
 		if (Constants.show_grid) drawGrid((Graphics2D) g);
-
-		// Graphics2D g2d = (Graphics2D) g;
-		// g2d.setStroke(Constants.getStroke(1, 1));
-		//		
-		// // g2d.setColor(Color.BLUE);
-		// // int w = handler.getDrawPanel().getScrollPanel().getViewport().getViewPosition().x;
-		// // int h = handler.getDrawPanel().getScrollPanel().getViewport().getViewPosition().y;
-		// // System.out.println(w + "," + h);
-		// // System.out.println(_scr.getHorizontalScrollBar().getValue() + "/ " + _scr.getVerticalScrollBar().getValue());
-		// // g2d.drawRect(w, h, (int) w+2, h+2);
-		// // g2d.setColor(Color.GRAY);
-		// // Dimension dim = getViewableDiagrampanelSize();
-		// // g2d.drawRect(0, 0, (int) dim.getWidth(), (int) dim.getHeight());
-		// // g2d.setColor(Color.RED);
-		// // dim = getPreferredSize();
-		// // g2d.drawRect(0, 0, (int) dim.getWidth(), (int) dim.getHeight());
-		//				
-		// g2d.setStroke(Constants.getStroke(0, 1));
-		// this.repaint();
-
 		super.paintComponents(g);
 	}
 
+	public void showNotification(String message) {
+
+		// There are no notifications on palette panels
+		if (getHandler() instanceof PaletteHandler) return;
+
+		if (!notifications.isEmpty()) {
+			for (DiagramNotification n : notifications)
+				this.remove(n);
+		}
+
+		DiagramNotification notification = new DiagramNotification(this.getHandler(), message);
+		notifications.add(notification);
+		this.add(notification);
+	}
+
+	/**
+	 * AB: Returns a copy of the actual origin zoomed to 100%.
+	 * 
+	 * Origin marks a point that tracks changes of the diagram panel size and can
+	 * be used to regenerate the original position of entities at the time they have been cut/copied,
+	 * etc...
+	 * 
+	 * @return a point that marks the diagram origin at a zoom of 100%.
+	 */
+
+	public Point getOriginAtDefaultZoom()
+	{
+		Point originCopy = new Point(origin);
+		originCopy.setLocation(
+				(origin.x * Constants.DEFAULTGRIDSIZE) / handler.getGridSize(),
+				(origin.y * Constants.DEFAULTGRIDSIZE) / handler.getGridSize());
+		return originCopy;
+	}
+	
+	/**
+	 * AB: Returns a copy of the actual origin.
+	 * Origin marks a point that tracks changes of the diagram panel size and can
+	 * be used to regenerate the original position of entities at the time they have been cut/copied,
+	 * etc...
+	 * 
+	 * @return a point that marks the diagram origin.
+	 */
+	public Point getOrigin() {
+		log.debug("Diagram origin: " + origin);
+		return new Point(origin);
+	}
+
+	
+	/**
+	 * AB: Moves the origin around the given delta x and delta y
+	 * 
+	 * This method is mainly used by updatePanelAndScrollBars() to keep track of the panels size changes. 
+	 */	
+	public void moveOrigin(int dx, int dy) {
+		log.debug("Move origin to: " + origin);
+		this.origin.translate(handler.realignToGrid(false, dx), handler.realignToGrid(false, dy));
+	}
+
+	/**
+	 * AB: Zoom the origin from the old grid size to the new grid size
+	 * 
+	 * this method is mainly used by the DiagramHandler classes setGridAndZoom method. 
+	 * 
+	 * @see DiagramHandler.setGridAndZoom
+	 * 	
+	 * @param oldGridSize the old grid size
+	 * @param newGridSize the new grid size
+	 */
+	public void zoomOrigin(int oldGridSize, int newGridSize) {
+		log.debug("Zoom origin to: " + origin);
+		origin.setLocation((origin.x * newGridSize) / oldGridSize, (origin.y * newGridSize) / oldGridSize);
+	}
 }
