@@ -17,7 +17,7 @@ public class ErrorOccurred extends Entity {
 		g2.setColor(Color.red);
 		g2.setFont(this.getHandler().getZoomedFont());
 		this.getHandler().writeText(g2, "This Element has an Error!", this.getWidth() / 2, this.getHeight() / 2 - (int) (10 * zoom), true);
-		g2.setColor(_activeColor);
+		g2.setColor(activeColor);
 	}
 
 }
