@@ -2,6 +2,7 @@
 package com.umlet.element.base;
 
 import java.awt.Color;
+import java.awt.Dimension;
 import java.awt.Font;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
@@ -11,6 +12,7 @@ import java.awt.Rectangle;
 import java.awt.font.TextLayout;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Area;
+import java.awt.geom.Rectangle2D;
 import java.util.Vector;
 
 import com.umlet.constants.Constants;
@@ -186,8 +188,6 @@ public class Relation extends Entity {
 			}
 			else if (s.startsWith("lt=") & (s.length() > 3)) {
 
-				// Mueller G. Beginn
-
 				csdStartText = "";
 				csdEndText = "";
 
@@ -217,9 +217,7 @@ public class Relation extends Entity {
 					endArrow = endArrow + "del";
 				}
 
-				// Mueller G. End
-
-				if (s.indexOf(">>>>>") >= 0) { // LME_A
+				if (s.indexOf(">>>>>") >= 0) {
 					endArrow = "<<<";
 				}
 				else if (s.indexOf(">>>>") >= 0) {
@@ -231,11 +229,14 @@ public class Relation extends Entity {
 				else if (s.indexOf(">>") >= 0) {
 					endArrow = "<<";
 				}
+				// else if (s.indexOf("m>") >= 0) {
+				// endArrow = "crowsFoot";
+				// }
 				else if (s.indexOf(">") >= 0) {
 					if (endArrow.equals("")) endArrow = "<";
 				}
 
-				if (s.indexOf("<<<<<") >= 0) { // LME: filled arrow head
+				if (s.indexOf("<<<<<") >= 0) {
 					beginArrow = "<<<";
 				}
 				else if (s.indexOf("<<<<") >= 0) {
@@ -247,6 +248,9 @@ public class Relation extends Entity {
 				else if (s.indexOf("<<") >= 0) {
 					beginArrow = "<<";
 				}
+				// else if (s.indexOf("<m") >= 0) {
+				// beginArrow = "crowsFoot";
+				// }
 				else if (s.indexOf("<") >= 0) {
 					if (beginArrow.equals("")) beginArrow = "<";
 				}
@@ -610,6 +614,22 @@ public class Relation extends Entity {
 		return true;
 	}
 
+	/**
+	 * A relation is only in range (= in the selection frame) if every relation point is in the selection frame
+	 */
+	@Override
+	public boolean isInRange(Point upperLeft, Dimension size) {
+		Rectangle2D rect1 = new Rectangle2D.Double(upperLeft.getX(), upperLeft.getY(), size.getWidth(), size.getHeight());
+		// Assume that the rect contains all relation points
+		for (Point p : getLinePoints()) {
+			// We must add the displacement from the top left corner of the drawpanel to the point coordinates
+			Point realPoint = new Point((int) (p.getX() + this.getX()), (int) (p.getY() + this.getY()));
+			// If only one point is not in the selection rectangle, the method returns false
+			if (!rect1.contains(realPoint)) return false;
+		}
+		return true;
+	}
+
 	public boolean isOnLine(int i) {
 		if ((i - 1 >= 0) & (i + 1 < getLinePoints().size())) {
 			Point x1 = getLinePoints().elementAt(i - 1);
@@ -716,6 +736,8 @@ public class Relation extends Entity {
 
 		c.setVisible(true);
 		c.setBounds(this.getBounds());
+		c.setHandler(this.getHandler());
+
 		return c;
 	}
 
@@ -781,8 +803,8 @@ public class Relation extends Entity {
 
 		Graphics2D g2 = (Graphics2D) g;
 		g2.setFont(this.getHandler().getZoomedFont());
-		g2.setColor(_activeColor);
-		this.getHandler().getFRC(g2); // Just to set anti-aliasing, even if no text
+		g2.setColor(activeColor);
+		Constants.getFRC(g2); // Just to set anti-aliasing, even if no text
 		// operations occur
 
 		// g2.setColor(Color.MAGENTA);
@@ -790,7 +812,7 @@ public class Relation extends Entity {
 		// g2.drawLine(0,0,2,0);
 		// g2.drawLine(this.getWidth()-1-2,this.getHeight()-1, this.getWidth()-1,this.getHeight()-1);
 		// g2.drawLine(this.getWidth()-1,this.getHeight()-1-2, this.getWidth()-1,this.getHeight()-1);
-		// g2.setColor(_activeColor);
+		// g2.setColor(activeColor);
 
 		Vector<Rectangle> startShapes = new Vector<Rectangle>();
 		Vector<Rectangle> endShapes = new Vector<Rectangle>();
@@ -799,7 +821,7 @@ public class Relation extends Entity {
 		endShapes.add(new NoShape());
 
 		if ((beginQualifier != null) && (beginQualifier.length() > 0)) {
-			TextLayout tl = new TextLayout(beginQualifier, this.getHandler().getZoomedFont(), this.getHandler().getFRC(g2));
+			TextLayout tl = new TextLayout(beginQualifier, this.getHandler().getZoomedFont(), Constants.getFRC(g2));
 			Qualifier q = new Qualifier(beginQualifier, 0, 0, (int) tl.getBounds().getWidth()
 					+ (int) this.getHandler().getZoomedFontsize() * 2, (int) tl.getBounds().getHeight()
 					+ (int) this.getHandler().getZoomedFontsize() / 2);
@@ -807,7 +829,7 @@ public class Relation extends Entity {
 		}
 		if ((endQualifier != null) && (endQualifier.length() > 0)) {
 			TextLayout tl = new TextLayout(endQualifier, this.getHandler().getZoomedFont(),
-					this.getHandler().getFRC(g2));
+					Constants.getFRC(g2));
 			Qualifier q = new Qualifier(endQualifier, 0, 0, (int) tl.getBounds().getWidth()
 					+ (int) this.getHandler().getZoomedFontsize() * 2, (int) tl.getBounds().getHeight()
 					+ (int) this.getHandler().getZoomedFontsize() / 2);
@@ -824,7 +846,7 @@ public class Relation extends Entity {
 		if ((beginMultiplicity != null) && (beginMultiplicity.length() > 0)) {
 			EmptyShape e = new EmptyShape((int) this.getHandler().getZoomedFontsize());
 			startShapes.add(e);
-			TextLayout tl = new TextLayout(beginMultiplicity, this.getHandler().getZoomedFont(), this.getHandler().getFRC(g2));
+			TextLayout tl = new TextLayout(beginMultiplicity, this.getHandler().getZoomedFont(), Constants.getFRC(g2));
 			Multiplicity m = new Multiplicity(beginMultiplicity, 0, 0, (int) tl.getBounds().getWidth(),
 					(int) tl.getBounds().getHeight());
 			startShapes.add(m);
@@ -832,7 +854,7 @@ public class Relation extends Entity {
 		if ((endMultiplicity != null) && (endMultiplicity.length() > 0)) {
 			EmptyShape e = new EmptyShape((int) this.getHandler().getZoomedFontsize());
 			endShapes.add(e);
-			TextLayout tl = new TextLayout(endMultiplicity, this.getHandler().getZoomedFont(), this.getHandler().getFRC(g2));
+			TextLayout tl = new TextLayout(endMultiplicity, this.getHandler().getZoomedFont(), Constants.getFRC(g2));
 			Multiplicity m = new Multiplicity(endMultiplicity, 0, 0, (int) tl.getBounds().getWidth(),
 					(int) tl.getBounds().getHeight());
 			endShapes.add(m);
@@ -854,7 +876,7 @@ public class Relation extends Entity {
 
 					broadestText = Math.max(broadestText, this.getHandler()
 							.getZoomedTextWidth(g2, beginRole.substring(position,
-							positionNew)));
+									positionNew)));
 					if (beginRole.lastIndexOf("\\\\") + 2 != beginRole.length()) broadestText = Math.max(broadestText, this.getHandler().getZoomedTextWidth(g2,
 							beginRole.substring(beginRole.lastIndexOf("\\\\") + 2, beginRole.length())));
 					lineBreaks++;
@@ -869,7 +891,7 @@ public class Relation extends Entity {
 			// <OLDCODE>
 			/*
 			 * TextLayout tl = new TextLayout(beginRole, this.getHandler().getFont(),
-			 * this.getHandler().getFRC(g2)); Role r = new Role(beginRole, 0, 0, (int)
+			 * Constants.getFRC(g2)); Role r = new Role(beginRole, 0, 0, (int)
 			 * tl.getBounds().getWidth(), (int) tl.getBounds().getHeight());
 			 */
 			// </OLDCODE>
@@ -894,7 +916,7 @@ public class Relation extends Entity {
 							endRole.substring(position, positionNew)));
 					if (endRole.lastIndexOf("\\\\") + 2 != endRole.length()) broadestText = Math.max(broadestText,
 							this.getHandler().getZoomedTextWidth(g2, endRole.substring(endRole.lastIndexOf("\\\\") + 2,
-							endRole.length())));
+									endRole.length())));
 					lineBreaks++;
 				}
 
@@ -907,7 +929,7 @@ public class Relation extends Entity {
 			// <OLDCODE>
 			/*
 			 * TextLayout tl = new TextLayout(endRole, this.getHandler().getFont(),
-			 * this.getHandler().getFRC(g2)); Role r = new Role(endRole, 0, 0, (int)
+			 * Constants.getFRC(g2)); Role r = new Role(endRole, 0, 0, (int)
 			 * tl.getBounds().getWidth(), (int) tl.getBounds().getHeight());
 			 */
 			// </OLDCODE>
@@ -919,7 +941,7 @@ public class Relation extends Entity {
 
 			EmptyShape e = new EmptyShape((int) this.getHandler().getZoomedFontsize());
 			startShapes.add(e);
-			TextLayout tl = new TextLayout(beginPort, this.getHandler().getZoomedFont(), this.getHandler().getFRC(g2));
+			TextLayout tl = new TextLayout(beginPort, this.getHandler().getZoomedFont(), Constants.getFRC(g2));
 			Port p = new Port(beginPort, 0, 0, (int) tl.getBounds().getWidth(), (int) tl.getBounds().getHeight());
 
 			startShapes.add(p);
@@ -928,7 +950,7 @@ public class Relation extends Entity {
 		if ((endPort != null) && (endPort.length() > 0)) {
 			EmptyShape e = new EmptyShape((int) this.getHandler().getZoomedFontsize());
 			endShapes.add(e);
-			TextLayout tl = new TextLayout(endPort, this.getHandler().getZoomedFont(), this.getHandler().getFRC(g2));
+			TextLayout tl = new TextLayout(endPort, this.getHandler().getZoomedFont(), Constants.getFRC(g2));
 			Port p = new Port(endPort, 0, 0, (int) tl.getBounds().getWidth(), (int) tl.getBounds().getHeight());
 			endShapes.add(p);
 		}
@@ -982,7 +1004,7 @@ public class Relation extends Entity {
 							px2.y);
 					at2.rotate(winkel, cx, cy);
 					g2.setTransform(at2);
-					g2.setColor(_activeColor);
+					g2.setColor(activeColor);
 					g2.setStroke(Constants.getStroke(0, 2));
 					g2.drawArc(mitte.x, mitte.y - (int) (10 * zoom), (int) (20 * zoom), (int) (20 * zoom), 90, 180);
 					g2.setStroke(Constants.getStroke(0, 1));
@@ -1018,23 +1040,23 @@ public class Relation extends Entity {
 					g2.setTransform(at2);
 
 					if (eerRelDir.equals("EER1")) {
-						g2.setColor(_activeColor);
+						g2.setColor(activeColor);
 						g2.fillPolygon(po1);
 						g2.fillPolygon(po2);
 					}
 					else if (eerRelDir.equals("EER2")) {
-						g2.setColor(_fillColor);
+						g2.setColor(fillColor);
 						g2.fillPolygon(po2);
-						g2.setColor(_activeColor);
+						g2.setColor(activeColor);
 						g2.fillPolygon(po1);
 					}
 					else if (eerRelDir.equals("EER3")) {
-						g2.setColor(_fillColor);
+						g2.setColor(fillColor);
 						g2.fillPolygon(po1);
-						g2.setColor(_activeColor);
+						g2.setColor(activeColor);
 						g2.fillPolygon(po2);
 					}
-					g2.setColor(_activeColor);
+					g2.setColor(activeColor);
 					g2.draw(po1);
 					g2.draw(po2);
 					g2.setTransform(at);
@@ -1061,7 +1083,7 @@ public class Relation extends Entity {
 
 					g2.setColor(Color.white);
 					g2.fillOval(outerArc.x, outerArc.y, (int) (30 * zoom), (int) (30 * zoom));
-					g2.setColor(_activeColor);
+					g2.setColor(activeColor);
 					g2.setStroke(Constants.getStroke(0, 1));
 
 					if (clientServer.equals("provideRequire")) {
@@ -1097,7 +1119,7 @@ public class Relation extends Entity {
 					if (middleArrow.equals("delegation")) {
 						g2.setColor(Color.white);
 						g2.fillOval(circle.x + (int) (5 * zoom), circle.y + (int) (5 * zoom), (int) (20 * zoom), (int) (20 * zoom));
-						g2.setColor(_activeColor);
+						g2.setColor(activeColor);
 						g2.setStroke(Constants.getStroke(0, 1));
 						g2.drawOval(circle.x + (int) (5 * zoom), circle.y + (int) (5 * zoom), (int) (20 * zoom), (int) (20 * zoom));
 					}
@@ -1114,7 +1136,7 @@ public class Relation extends Entity {
 
 						g2.setColor(Color.white);
 						g2.fillOval(circle.x + (int) (5 * zoom), circle.y + (int) (5 * zoom), (int) (20 * zoom), (int) (20 * zoom));
-						g2.setColor(_activeColor);
+						g2.setColor(activeColor);
 						g2.setStroke(Constants.getStroke(0, 1));
 						g2.drawOval(circle.x + (int) (5 * zoom), circle.y + (int) (5 * zoom), (int) (20 * zoom), (int) (20 * zoom));
 
@@ -1122,14 +1144,14 @@ public class Relation extends Entity {
 					if (middleArrow.equals("delegationStart")) {
 						g2.setColor(Color.white);
 						g2.fillArc(circle.x, circle.y, (int) (30 * zoom), (int) (30 * zoom), 90, 180);
-						g2.setColor(_activeColor);
+						g2.setColor(activeColor);
 						g2.setStroke(Constants.getStroke(0, 1));
 						g2.drawArc(circle.x, circle.y, (int) (30 * zoom), (int) (30 * zoom), 90, 180);
 					}
 					if (middleArrow.equals("delegationEnd")) {
 						g2.setColor(Color.white);
 						g2.fillArc(circle.x, circle.y, (int) (30 * zoom), (int) (30 * zoom), 90, -180);
-						g2.setColor(_activeColor);
+						g2.setColor(activeColor);
 						g2.setStroke(Constants.getStroke(0, 1));
 						g2.drawArc(circle.x, circle.y, (int) (30 * zoom), (int) (30 * zoom), 90, -180);
 					}
@@ -1177,9 +1199,9 @@ public class Relation extends Entity {
 				Qualifier q = (Qualifier) r;
 
 				// begin B. Buckl
-				g.setColor(_fillColor);
+				g.setColor(fillColor);
 				g.fillRect((int) r.getX(), (int) r.getY(), (int) r.getWidth(), (int) r.getHeight());
-				g.setColor(_activeColor);
+				g.setColor(activeColor);
 				// end
 				g.drawRect((int) r.getX(), (int) r.getY(), (int) r.getWidth(), (int) r.getHeight());
 				this.getHandler().writeText(g2, q.getString(), (int) r.getX()
@@ -1189,8 +1211,17 @@ public class Relation extends Entity {
 			}
 			else if (r instanceof Arrow) {
 				Arrow arrow = (Arrow) r;
-				// A.Mueller Start
-				if (!arrow.getString().equals("n")
+
+				if (arrow.getString().equals("crowsFoot")) {
+
+					g2.drawLine((int) arrow.getX(), (int) arrow.getY() + arrow.getArrowEndA().y,
+							(int) arrow.getX() + arrow.getArrowEndA().x, (int) arrow.getY());
+					g2.drawLine((int) arrow.getX(), (int) arrow.getY() + arrow.getArrowEndB().y,
+							(int) arrow.getX() + arrow.getArrowEndB().x, (int) arrow.getY());
+
+					// A.Mueller Start
+				}
+				else if (!arrow.getString().equals("n")
 						&& !arrow.getString().equals("require")
 						&& !arrow.getString().equals("provide")
 						&& !arrow.getString().startsWith("package")
@@ -1233,9 +1264,9 @@ public class Relation extends Entity {
 					ay[1] = (int) arrow.getY() + arrow.getArrowEndA().y;
 					ay[2] = (int) arrow.getY() + arrow.getArrowEndB().y;
 					Polygon myPg = new Polygon(ax, ay, 3);
-					g2.setColor(_fillColor);
+					g2.setColor(fillColor);
 					g2.fill(myPg);
-					g2.setColor(_activeColor);
+					g2.setColor(activeColor);
 					g2.draw(myPg);
 
 					// g2.drawLine((int)arrow.getX()+(int)arrow.getArrowEndA().x,
@@ -1258,9 +1289,9 @@ public class Relation extends Entity {
 
 					// begin B. Buckl
 					Polygon myPg = new Polygon(ax, ay, 4);
-					g2.setColor(_fillColor);
+					g2.setColor(fillColor);
 					g2.fill(myPg);
-					g2.setColor(_activeColor);
+					g2.setColor(activeColor);
 					g2.draw(myPg);
 					// end B. Buckl
 				}
@@ -1303,7 +1334,7 @@ public class Relation extends Entity {
 				else if (arrow.getString().equals("provide")) {
 					int width = arrow.getCrossEndB().x - arrow.getCrossEndA().x;
 					int height = arrow.getCrossEndB().y
-							- arrow.getCrossEndA().y;
+					- arrow.getCrossEndA().y;
 					g2.drawArc((int) arrow.getX() + arrow.getCrossEndA().x, (int) arrow.getY() + arrow.getCrossEndA().y, width, height, arrow.getArcStart(), arrow.getArcEnd());
 					// A.Mueller End
 					// G.Mueller Start
@@ -1327,9 +1358,9 @@ public class Relation extends Entity {
 					double winkel = Constants.getAngle(px1.x, px1.y, px2.x, px2.y);
 					at2.rotate(winkel, cx, cy);
 					g2.setTransform(at2);
-					g2.setColor(_fillColor);
+					g2.setColor(fillColor);
 					g2.fillOval(px1.x, px1.y - (int) (10 * zoom), (int) (20 * zoom), (int) (20 * zoom));
-					g2.setColor(_activeColor);
+					g2.setColor(activeColor);
 					g2.drawOval(px1.x, px1.y - (int) (10 * zoom), (int) (20 * zoom), (int) (20 * zoom));
 					g2.drawLine(px1.x + (int) (10 * zoom), px1.y - (int) (5 * zoom), px1.x + (int) (10 * zoom), px1.y + (int) (5 * zoom));
 					g2.drawLine(px1.x + (int) (15 * zoom), px1.y, px1.x + (int) (5 * zoom), px1.y);
@@ -1396,9 +1427,9 @@ public class Relation extends Entity {
 
 						px1 = getStartPoint();
 						px2 = getLinePoints().elementAt(1);
-						g2.setColor(_fillColor);
+						g2.setColor(fillColor);
 						g2.fillRect(px1.x - s / 2, px1.y - s / 2, s, s);
-						g2.setColor(_activeColor);
+						g2.setColor(activeColor);
 						g2.drawRect(px1.x - s / 2, px1.y - s / 2, s, s);
 						g2.setFont(new Font("SansSerif", Font.PLAIN, (int) (10 * zoom)));
 						if (csdStartText.equals(">")) {
@@ -1459,9 +1490,9 @@ public class Relation extends Entity {
 						px1 = getEndPoint();
 						px2 = getLinePoints().elementAt(
 								getLinePoints().size() - 2);
-						g2.setColor(_fillColor);
+						g2.setColor(fillColor);
 						g2.fillRect(px1.x - s / 2, px1.y - s / 2, s, s);
-						g2.setColor(_activeColor);
+						g2.setColor(activeColor);
 						g2.drawRect(px1.x - s / 2, px1.y - s / 2, s, s);
 						g2.setFont(new Font("SansSerif", Font.PLAIN, (int) (10 * zoom)));
 						if (csdEndText.equals(">")) {
