@@ -24,6 +24,7 @@ import com.umlet.control.diagram.DiagramHandler;
 // An interaction represents a synchronous/asynchronous message
 // that is sent between two objects.
 class Interaction {
+
 	private int srcObj;
 	private boolean srcObjHasControl;
 	private int arrowKind; // 1=SYNC, 2= ASYNC, 3=EDGE, 4=FILLED
@@ -185,8 +186,8 @@ public class SequenceDiagram extends com.umlet.element.base.Entity {
 		// quality, etc. You should not have to change this.
 		Graphics2D g2 = (Graphics2D) g;
 		g2.setFont(this.getHandler().getZoomedFont());
-		g2.setColor(_activeColor);
-		this.getHandler().getFRC(g2);
+		g2.setColor(activeColor);
+		Constants.getFRC(g2);
 
 		// draw the border
 		g2.drawRect(0, 0, this.getWidth() - 1, this.getHeight() - 1);
@@ -220,6 +221,7 @@ public class SequenceDiagram extends com.umlet.element.base.Entity {
 				element = "9999<-0:" + element; // dummy: space for interactionframe
 				lines.set(i, element);
 			}
+			// AB: match whitespace characters from the beginning to the end of the line
 			if (lines.elementAt(i).matches("\\A\\s*\\z")) continue;
 			levelNum++;
 		}
@@ -232,6 +234,7 @@ public class SequenceDiagram extends com.umlet.element.base.Entity {
 		// get the labels of the Sequencediagram
 		String newhead = ""; // delete the ids from the header
 		this.labeltonumber = new HashMap<String, Integer>();
+
 		Pattern p_label = Pattern.compile("([^\\~]+)(\\~([a-zA-Z0-9]+))?(\\_)?");
 		for (int i = 1; i <= numObjects; i++) {
 			Matcher m = p_label.matcher(obj.get(i - 1));
@@ -239,9 +242,13 @@ public class SequenceDiagram extends com.umlet.element.base.Entity {
 				this.labeltonumber.put(m.group(3), i);
 				newhead += "|" + m.group(1) + (m.group(4) == null ? "" : m.group(4));
 			}
-			else newhead += "|" + obj.get(i - 1);
+			else {
+				newhead += "|" + obj.get(i - 1);
+			}
 			if (!this.labeltonumber.containsKey(Integer.toString(i))) // only write number if no other label has this number
-			this.labeltonumber.put(Integer.toString(i), i); // columnnumber as label for backward compatibility
+			{
+				this.labeltonumber.put(Integer.toString(i), i); // columnnumber as label for backward compatibility
+			}
 		}
 		obj = Constants.decomposeStrings(newhead.length() > 0 ? newhead.substring(1) : "", "|");
 		// LABELADDING STOP (exchanged parseInteger Methods with labeltonumber.get methods
@@ -251,18 +258,21 @@ public class SequenceDiagram extends com.umlet.element.base.Entity {
 		im = new InteractionManagement(levelNum);
 		String boxStrings = "";
 		for (int i = 1; i < lines.size(); i++) {
+			String methodName = "";
 			if (lines.elementAt(i).matches("\\A\\s*\\z")) continue;
 			curLevel++;
 			Vector<String> interactions = Constants.decomposeStrings(lines.elementAt(i), ";");
 
 			for (int j = 0; j < interactions.size(); j++) {
-				Pattern p =
-										// Pattern.compile("\\A(\\d+)(->>|->|-/>|.>>|.>|./>|->>>|.>>>)(\\d+)(:((\\d+)(,(\\d+))*))*(?::(.*))?\\z");
-				Pattern.compile("\\A(\\w+)(->>|->|-/>|.>>|.>|./>|->>>|.>>>|<<-|<-|</-|<<.|<.|</.|<<<-|<<<.)(\\w+)(:((\\w+)(,(\\w+))*))?(?::(.*))?\\z");
+				Pattern p = Pattern.compile("\\A(\\w+)(->>|->|-/>|.>>|.>|./>|->>>|.>>>|<<-|<-|</-|<<.|<.|</.|<<<-|<<<.)(\\w+)(:((\\w+)(,(\\w+))*))?(?::(.*))?\\z");
+				// Pattern.compile("\\A(\\d+)(->>|->|-/>|.>>|.>|./>|->>>|.>>>)(\\d+)(:((\\d+)(,(\\d+))*))*(?::(.*))?\\z");
+
 				// 1->2:1,2:methodName
 				// 1->2:abc
 				Matcher m = p.matcher(interactions.elementAt(j));
+
 				if (!m.matches()) continue;
+
 				Integer srcObj = this.labeltonumber.get(m.group(1));
 				Integer destObj = this.labeltonumber.get(m.group(3));
 
@@ -356,13 +366,24 @@ public class SequenceDiagram extends com.umlet.element.base.Entity {
 						Integer tempgroup = this.labeltonumber.get(tmp);
 						if (tempgroup != null) group += "," + tempgroup;
 					}
-					if (group.length() > 0) group = group.substring(1);
+					if (group.length() > 0) {
+						group = group.substring(1);
+					}
+					else {
+						if (grouparray.length == 1) {
+							group = "#";
+							methodName = grouparray[0];
+						}
+					}
 				} // STOP LABLING ADD
 				boxStrings += ";" + group; // LME: get alive Objects
 
 				boolean srcObjHasControl = srcObj != null ? group.contains(String.valueOf(srcObj)) : false;
 				boolean destObjHasControl = destObj != null ? group.contains(String.valueOf(destObj)) : false;
-				String methodName = m.group(9);
+
+				if (methodName.isEmpty()) {
+					methodName = m.group(9);
+				}
 
 				// LME: removed (in V6) since not necessary
 				// if(destObj==srcObj) levelNum++; //LME: expand the Entity's size
@@ -478,7 +499,7 @@ public class SequenceDiagram extends com.umlet.element.base.Entity {
 						int xs[] = { p1.x, d1.x, d2.x };
 						int ys[] = { p1.y, d1.y, d2.y };
 						Color oldColor = g2.getColor();
-						g2.setColor(_fillColor);
+						g2.setColor(fillColor);
 						g2.fillPolygon(xs, ys, 3);
 						g2.setColor(oldColor);
 						g2.drawPolygon(xs, ys, 3);
@@ -549,7 +570,7 @@ public class SequenceDiagram extends com.umlet.element.base.Entity {
 			boolean centerVertically, boolean centerHorizontically) {
 
 		if ((methodName == null) || methodName.equals("")) {
-			System.err.println("SequenceDiagram->printMethodName was called with an invalid argument.");
+			log.error("SequenceDiagram->printMethodName was called with an invalid argument.");
 			return 0;
 		}
 
