@@ -4,8 +4,11 @@ package com.umlet.element.custom;
 import java.awt.Composite;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
+import java.awt.Point;
 import java.awt.Polygon;
 
+import com.umlet.constants.Constants;
+import com.umlet.control.diagram.StickingPolygon;
 import com.umlet.element.base.Entity;
 
 @SuppressWarnings("serial")
@@ -15,8 +18,8 @@ public class Decision extends Entity {
 		Graphics2D g2 = (Graphics2D) g;
 		g2.setFont(this.getHandler().getZoomedFont());
 		Composite[] composites = colorize(g2); // enable colors
-		g2.setColor(_activeColor);
-		this.getHandler().getFRC(g2);
+		g2.setColor(activeColor);
+		Constants.getFRC(g2);
 
 		Polygon poly = new Polygon();
 		poly.addPoint(this.getWidth() / 2, 0);
@@ -25,18 +28,25 @@ public class Decision extends Entity {
 		poly.addPoint(0, this.getHeight() / 2);
 
 		g2.setComposite(composites[1]);
-		g2.setColor(_fillColor);
+		g2.setColor(fillColor);
 		g2.fillPolygon(poly);
 		g2.setComposite(composites[0]);
-		if (_selected) g2.setColor(_activeColor);
-		else g2.setColor(_deselectedColor);
+		if (_selected) g2.setColor(activeColor);
+		else g2.setColor(deselectedColor);
 
 		g2.drawPolygon(poly);
+	}
 
-		// g2.drawLine(this.getWidth()/2,0,this.getWidth(),this.getHeight()/2);
-		// g2.drawLine(this.getWidth()/2,0,0,this.getHeight()/2);
-		// g2.drawLine(0,this.getHeight()/2,this.getWidth()/2,this.getHeight());
-		// g2.drawLine(this.getWidth()-1,this.getHeight()/2,this.getWidth()/2-1,this.getHeight());
+	@Override
+	public StickingPolygon generateStickingBorder(int x, int y, int width, int height) {
+		StickingPolygon p = new StickingPolygon();
+		y += 1;
+		width += 1;
+		p.addPoint(new Point(x + width / 2, y));
+		p.addPoint(new Point(x + width, y + height / 2));
+		p.addPoint(new Point(x + width / 2, y + height));
+		p.addPoint(new Point(x, y + height / 2), true);
+		return p;
 	}
 
 	@Override
