@@ -19,7 +19,7 @@ public class InitialFinalState extends Entity {
 		Graphics2D g2 = (Graphics2D) g;
 		g2.setFont(this.getHandler().getZoomedFont());
 		g2.setColor(Color.red);
-		this.getHandler().getFRC(g2);
+		Constants.getFRC(g2);
 
 		Vector<String> tmp = Constants.decomposeStrings(this.getPanelAttributes(), "\n");
 
