@@ -5,6 +5,7 @@ import java.awt.Graphics;
 import java.awt.Graphics2D;
 import java.awt.geom.AffineTransform;
 
+import com.umlet.constants.Constants;
 import com.umlet.element.base.Entity;
 
 @SuppressWarnings("serial")
@@ -15,8 +16,8 @@ public class RegionEndState extends Entity {
 		Graphics2D g2 = (Graphics2D) g;
 		g2.setFont(this.getHandler().getZoomedFont());
 		colorize(g2); // enable colors
-		g2.setColor(_activeColor);
-		this.getHandler().getFRC(g2);
+		g2.setColor(activeColor);
+		Constants.getFRC(g2);
 
 		g2.drawOval(0, 0, this.getWidth() - 1, this.getHeight() - 1);
 		AffineTransform at = g2.getTransform();
