@@ -6,6 +6,7 @@ import java.awt.Graphics2D;
 import java.awt.Point;
 import java.awt.Rectangle;
 
+import com.umlet.constants.Constants;
 import com.umlet.control.diagram.StickingPolygon;
 import com.umlet.element.base.Entity;
 
@@ -15,8 +16,8 @@ public class SeqDestroyMark extends Entity {
 	public void paintEntity(Graphics g) {
 		Graphics2D g2 = (Graphics2D) g;
 		colorize(g2); // enable colors
-		g2.setColor(_activeColor);
-		this.getHandler().getFRC(g2);
+		g2.setColor(activeColor);
+		Constants.getFRC(g2);
 
 		Rectangle r = this.getBounds();
 		g2.drawLine(0, 0, (int) r.getWidth() - 1, (int) r.getHeight() - 1);
