@@ -14,15 +14,15 @@ public class SeqObjectActive extends Entity {
 	public void paintEntity(Graphics g) {
 		Graphics2D g2 = (Graphics2D) g;
 		Composite[] composites = colorize(g2); // enable colors
-		g2.setColor(_activeColor);
-		this.getHandler().getFRC(g2);
+		g2.setColor(activeColor);
+		Constants.getFRC(g2);
 
 		g2.setComposite(composites[1]);
-		g2.setColor(_fillColor);
+		g2.setColor(fillColor);
 		g2.fillRect(0, 0, this.getWidth() - 1, this.getHeight() - 1);
 		g2.setComposite(composites[0]);
-		if (_selected) g2.setColor(_activeColor);
-		else g2.setColor(_deselectedColor);
+		if (_selected) g2.setColor(activeColor);
+		else g2.setColor(deselectedColor);
 
 		g2.drawRect(0, 0, this.getWidth() - 1, this.getHeight() - 1);
 
