@@ -4,6 +4,8 @@ package com.umlet.element.custom;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
 
+import com.umlet.constants.Constants;
+
 @SuppressWarnings("serial")
 public class Socket extends com.umlet.element.base.Entity {
 
@@ -14,8 +16,8 @@ public class Socket extends com.umlet.element.base.Entity {
 
 		Graphics2D g2 = (Graphics2D) g;
 		g2.setFont(this.getHandler().getZoomedFont());
-		g2.setColor(_activeColor);
-		this.getHandler().getFRC(g2);
+		g2.setColor(activeColor);
+		Constants.getFRC(g2);
 
 		int fnt = (int) this.getHandler().getZoomedFontsize() + ((int) this.getHandler().getZoomedFontsize() / 2);
 		this.setSize(fnt + 2, fnt + 2);
