@@ -7,12 +7,16 @@ import javax.swing.JEditorPane;
 import javax.swing.JPanel;
 import javax.swing.border.LineBorder;
 
+import org.apache.log4j.Logger;
+
 import com.umlet.control.Umlet;
 import com.umlet.gui.base.listeners.HyperLinkActiveListener;
 
 @SuppressWarnings("serial")
 public class AboutPanel extends JPanel {
 
+	private static Logger log = Logger.getLogger(AboutPanel.class.getName());
+
 	private static AboutPanel window;
 
 	private AboutPanel() {
@@ -43,7 +47,7 @@ public class AboutPanel extends JPanel {
 			edit.setEditable(false);
 			edit.setSelectionColor(Color.WHITE);
 		} catch (Exception e) {
-			e.printStackTrace();
+			log.error(null, e);
 		}
 	}
 
