@@ -35,11 +35,15 @@ import javax.swing.JTextArea;
 import javax.swing.JTextField;
 import javax.swing.UIManager;
 
+import org.apache.log4j.Logger;
+
 import com.umlet.constants.Constants;
 import com.umlet.control.Umlet;
 
 public class MailPanel extends JPanel {
 
+	private static Logger log = Logger.getLogger(MailPanel.class.getName());
+
 	private static final long serialVersionUID = 1L;
 
 	/**
@@ -343,7 +347,7 @@ public class MailPanel extends JPanel {
 		}
 
 		catch (Exception e) {
-			e.printStackTrace();
+			log.error(null, e);
 			if (e instanceof MessagingException) { // SMTP Error
 				JOptionPane.showMessageDialog(this, "There has been an error with your smtp server." + Constants.NEWLINE + "Please recheck your smtp server and login data.", "SMTP Error", JOptionPane.ERROR_MESSAGE, UIManager.getIcon("OptionPane.errorIcon"));
 			}
