@@ -25,11 +25,11 @@ public class ScrollbarListener implements MouseListener, MouseWheelListener {
 	public void mousePressed(MouseEvent e) {}
 
 	public void mouseReleased(MouseEvent e) {
-		drawpanel.removeUnnecessaryWhitespaceAroundDiagram();
+		drawpanel.updatePanelAndScrollbars();
 	}
 
 	public void mouseWheelMoved(MouseWheelEvent e) {
-		drawpanel.removeUnnecessaryWhitespaceAroundDiagram();
+		drawpanel.updatePanelAndScrollbars();
 	}
 
 }
