@@ -17,7 +17,6 @@ import javax.swing.Box;
 import javax.swing.BoxLayout;
 import javax.swing.DefaultComboBoxModel;
 import javax.swing.ImageIcon;
-import javax.swing.JButton;
 import javax.swing.JComboBox;
 import javax.swing.JComponent;
 import javax.swing.JDialog;
@@ -33,10 +32,13 @@ import javax.swing.JScrollPane;
 import javax.swing.JSplitPane;
 import javax.swing.JTabbedPane;
 import javax.swing.JTextField;
+import javax.swing.JToggleButton;
 import javax.swing.KeyStroke;
 import javax.swing.ToolTipManager;
 import javax.swing.WindowConstants;
 
+import org.apache.log4j.Logger;
+
 import com.umlet.constants.Constants;
 import com.umlet.control.Umlet;
 import com.umlet.control.diagram.CustomPreviewHandler;
@@ -50,10 +52,15 @@ import com.umlet.gui.base.MailPanel;
 import com.umlet.gui.base.TabComponent;
 import com.umlet.gui.base.UmletGUI;
 import com.umlet.gui.base.UmletTextPane;
+import com.umlet.gui.base.listeners.DividerListener;
+import com.umlet.gui.base.listeners.GUIListener;
+import com.umlet.gui.base.listeners.PaletteComboBoxListener;
 
 @SuppressWarnings("serial")
 public class StandaloneGUI extends UmletGUI {
 
+	private static Logger log = Logger.getLogger(StandaloneGUI.class);
+
 	private JComboBox zoomComboBox;
 	private JFrame window;
 	private JSplitPane customSplit;
@@ -61,12 +68,14 @@ public class StandaloneGUI extends UmletGUI {
 	private JSplitPane mainSplit;
 	private JSplitPane mailSplit;
 	private JPanel diagramspanel;
-	private JMenu menu_palettes;
+	// private JMenu menu_palettes;
 	private JMenu menu_file;
 	private JPanel palettepanel;
 	private JTabbedPane diagramtabs;
 	private JMenuItem menu_file_save;
 	private JMenuItem menu_file_save_as;
+	private JMenu menu_file_export_as;
+	private JMenuItem palette_edit;
 	private JMenuItem menu_file_export_as_bmp;
 	private JMenuItem menu_file_export_as_eps;
 	private JMenuItem menu_file_export_as_gif;
@@ -74,15 +83,20 @@ public class StandaloneGUI extends UmletGUI {
 	private JMenuItem menu_file_export_as_pdf;
 	private JMenuItem menu_file_export_as_png;
 	private JMenuItem menu_file_export_as_svg;
+	private JMenuItem menu_file_mail_to;
+	private JMenuItem menu_file_options;
+	private JMenuItem menu_file_print;
+	private JMenuItem menu_file_exit;
+	private JMenu menu_edit;
+	private JMenuItem menu_edit_undo;
+	private JMenuItem menu_edit_redo;
 	private JMenuItem menu_edit_delete;
+	private JMenuItem menu_edit_select_all;
 	private JMenuItem menu_edit_group;
 	private JMenuItem menu_edit_ungroup;
 	private JMenuItem menu_edit_cut;
 	private JMenuItem menu_edit_copy;
 	private JMenuItem menu_edit_paste;
-	private JMenu menu_edit;
-	private JMenu menu_export_as;
-	private JMenuItem menu_send_per_mail;
 	private JMenuItem menu_custom_new;
 	private JMenu menu_custom_template;
 	private JMenuItem menu_custom_edit;
@@ -90,6 +104,9 @@ public class StandaloneGUI extends UmletGUI {
 	private JTextField searchField;
 	private UmletTextPane propertyTextPane;
 	private ZoomListener zoomListener;
+	private JPanel rightPanel;
+	private JPanel paletteControlsPanel;
+	private JComboBox paletteList;
 
 	protected String selected_palette;
 
@@ -99,7 +116,7 @@ public class StandaloneGUI extends UmletGUI {
 	private boolean custom_panel_visible;
 
 	private MailPanel mailPanel;
-	private JButton mailButton;
+	private JToggleButton mailButton;
 	private boolean mail_panel_visible;
 
 	public StandaloneGUI(Umlet umlet) {
@@ -112,9 +129,7 @@ public class StandaloneGUI extends UmletGUI {
 
 	private void onDiagramOpened() {
 		DrawPanel p = this.getCurrentDiagram();
-		if (p != null) {
-			Umlet.getInstance().setCurrentDiagram(p.getHandler());
-		}
+		if (p != null) Umlet.getInstance().setCurrentDiagram(p.getHandler());
 		else Umlet.getInstance().setCurrentDiagram(null);
 	}
 
@@ -159,6 +174,8 @@ public class StandaloneGUI extends UmletGUI {
 	@Override
 	protected void init() {
 
+		this.addKeyListener(new GUIListener());
+
 		this.window = new JFrame();
 		this.window.setContentPane(this);
 		this.window.addWindowListener(new WindowListener());
@@ -166,6 +183,11 @@ public class StandaloneGUI extends UmletGUI {
 		this.window.setBounds(Constants.umlet_location.x, Constants.umlet_location.y, Constants.umlet_size.width, Constants.umlet_size.height);
 		this.window.setTitle("UMLet - Free UML Tool for Fast UML Diagrams");
 
+		/************************ SET UMLET ICON ************************/
+		String iconPath = Umlet.getInstance().getHomePath() + Umlet.getIconsPath() + "umlet_logo.jpg";
+		this.window.setIconImage(new ImageIcon(iconPath).getImage());
+		/*************************************************************/
+
 		/************************ CREATE PROP PANE *****************/
 		this.propertyTextPane = this.createPropertyTextPane();
 
@@ -180,6 +202,9 @@ public class StandaloneGUI extends UmletGUI {
 		this.window.setTitle("UMLet - Free UML Tool for Fast UML Diagrams");
 		/***************************************/
 
+		paletteList = new JComboBox();
+		paletteList.setMaximumRowCount(15);
+
 		/*********** CREATE MENU *****************/
 		JMenuBar menu = new JMenuBar();
 		StandaloneMenuListener mlistener = new StandaloneMenuListener();
@@ -220,35 +245,48 @@ public class StandaloneGUI extends UmletGUI {
 		menu_file_export_as_svg.addActionListener(mlistener);
 
 		// Add the submenu entries to the menupoint "Export as"
-		menu_export_as = new JMenu("Export as");
-		menu_export_as.add(menu_file_export_as_bmp);
-		menu_export_as.add(menu_file_export_as_eps);
-		menu_export_as.add(menu_file_export_as_gif);
-		menu_export_as.add(menu_file_export_as_jpg);
-		menu_export_as.add(menu_file_export_as_pdf);
-		menu_export_as.add(menu_file_export_as_png);
-		menu_export_as.add(menu_file_export_as_svg);
+		menu_file_export_as = new JMenu("Export as");
+		menu_file_export_as.add(menu_file_export_as_bmp);
+		menu_file_export_as.add(menu_file_export_as_eps);
+		menu_file_export_as.add(menu_file_export_as_gif);
+		menu_file_export_as.add(menu_file_export_as_jpg);
+		menu_file_export_as.add(menu_file_export_as_pdf);
+		menu_file_export_as.add(menu_file_export_as_png);
+		menu_file_export_as.add(menu_file_export_as_svg);
 
-		menu_send_per_mail = new JMenuItem("Mail to...");
-		menu_send_per_mail.setMnemonic(KeyEvent.VK_M);
-		menu_send_per_mail.addActionListener(mlistener);
-		menu_send_per_mail.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, Constants.CTRLMETA_MASK));
+		menu_file_mail_to = new JMenuItem("Mail to...");
+		menu_file_mail_to.setMnemonic(KeyEvent.VK_M);
+		menu_file_mail_to.addActionListener(mlistener);
+		menu_file_mail_to.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, Constants.CTRLMETA_MASK));
 
-		JMenuItem menu_file_exit = new JMenuItem("Exit");
+		menu_file_exit = new JMenuItem("Exit");
 		menu_file_exit.addActionListener(mlistener);
-		JMenuItem menu_file_options = new JMenuItem("Options...");
+		menu_file_options = new JMenuItem("Options...");
 		menu_file_options.addActionListener(mlistener);
-		JMenuItem menu_file_print = new JMenuItem("Print...");
+		menu_file_print = new JMenuItem("Print...");
 		menu_file_print.setMnemonic(KeyEvent.VK_P);
 		menu_file_print.addActionListener(mlistener);
 		menu_file_print.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, Constants.CTRLMETA_MASK));
 
+		// Create Palettes and Edit Palette menuitem
+		List<String> palettenames = this.umlet.getPaletteNames();
+		for (String palette : palettenames) {
+			this.paletteList.addItem(palette);
+		}
+		PaletteComboBoxListener pl = new PaletteComboBoxListener();
+		this.paletteList.addActionListener(pl);
+		this.paletteList.addMouseWheelListener(pl);
+		palette_edit = new JMenuItem("Edit Current Palette");
+		palette_edit.addActionListener(mlistener);
+
 		menu_file.add(menu_file_new);
 		menu_file.add(menu_file_open);
 		menu_file.add(menu_file_save);
 		menu_file.add(menu_file_save_as);
-		menu_file.add(menu_export_as);
-		menu_file.add(menu_send_per_mail);
+		menu_file.add(menu_file_export_as);
+		menu_file.add(menu_file_mail_to);
+		menu_file.addSeparator();
+		menu_file.add(palette_edit);
 		menu_file.addSeparator();
 		menu_file.add(menu_file_options);
 		menu_file.addSeparator();
@@ -260,17 +298,17 @@ public class StandaloneGUI extends UmletGUI {
 		menu_edit = new JMenu("Edit");
 		menu_edit.setMnemonic(KeyEvent.VK_E);
 
-		JMenuItem menu_edit_undo = new JMenuItem("Undo");
+		menu_edit_undo = new JMenuItem("Undo");
 		menu_edit_undo.addActionListener(mlistener);
 		menu_edit_undo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, Constants.CTRLMETA_MASK));
-		JMenuItem menu_edit_redo = new JMenuItem("Redo");
+		menu_edit_redo = new JMenuItem("Redo");
 		menu_edit_redo.addActionListener(mlistener);
 		menu_edit_redo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, Constants.CTRLMETA_MASK));
 		menu_edit_delete = new JMenuItem("Delete");
 		menu_edit_delete.addActionListener(mlistener);
 		menu_edit_delete.setAccelerator(KeyStroke.getKeyStroke("DELETE"));
 
-		JMenuItem menu_edit_select_all = new JMenuItem("Select all");
+		menu_edit_select_all = new JMenuItem("Select All");
 		menu_edit_select_all.addActionListener(mlistener);
 		menu_edit_select_all.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, Constants.CTRLMETA_MASK));
 		menu_edit_group = new JMenuItem("Group");
@@ -312,28 +350,12 @@ public class StandaloneGUI extends UmletGUI {
 		menu_file_export_as_pdf.setEnabled(false);
 		menu_file_export_as_png.setEnabled(false);
 		menu_file_export_as_svg.setEnabled(false);
-		menu_edit.setEnabled(false);
 		menu_edit_delete.setEnabled(false);
 		menu_edit_group.setEnabled(false);
 		menu_edit_cut.setEnabled(false);
 		menu_edit_paste.setEnabled(false);
 		menu_edit_ungroup.setEnabled(false);
 
-		// palette menu
-		PaletteMenuListener pmlistener = new PaletteMenuListener();
-		this.menu_palettes = new JMenu("Palettes");
-		this.menu_palettes.setMnemonic(KeyEvent.VK_P);
-		List<String> palettenames = this.umlet.getPaletteNames();
-		for (String palette : palettenames) {
-			JMenuItem palette_item = new JMenuItem(palette);
-			palette_item.addActionListener(pmlistener);
-			this.menu_palettes.add(palette_item);
-		}
-		JMenuItem palette_edit = new JMenuItem("Edit Current Palette");
-		palette_edit.addActionListener(mlistener);
-		this.menu_palettes.addSeparator();
-		this.menu_palettes.add(palette_edit);
-
 		// Custom Element Menu
 		JMenu menu_custom = new JMenu("Custom Elements");
 		menu_custom.setMnemonic(KeyEvent.VK_C);
@@ -376,7 +398,7 @@ public class StandaloneGUI extends UmletGUI {
 		menu_help_homepage.addActionListener(mlistener);
 		menu_help.add(menu_help_homepage);
 
-		JMenuItem menu_help_rateUmlet = new JMenuItem("Rate UMLet at EclipsePluginCentral...");
+		JMenuItem menu_help_rateUmlet = new JMenuItem("Rate UMLet at Eclipse Marketplace...");
 		menu_help_rateUmlet.addActionListener(mlistener);
 		menu_help.add(menu_help_rateUmlet);
 
@@ -390,13 +412,13 @@ public class StandaloneGUI extends UmletGUI {
 
 		menu.add(menu_file);
 		menu.add(menu_edit);
-		menu.add(this.menu_palettes);
 		menu.add(menu_custom);
 		menu.add(menu_help);
 
 		/* Search field */
 
 		JPanel searchPanel = new JPanel();
+		searchPanel.setOpaque(false);
 		searchPanel.setLayout(new BoxLayout(searchPanel, BoxLayout.X_AXIS));
 		JLabel searchLabel = new JLabel("Search:   ");
 		searchField = new JTextField(10);
@@ -415,6 +437,7 @@ public class StandaloneGUI extends UmletGUI {
 		/* Zoom field */
 
 		JPanel zoomPanel = new JPanel();
+		zoomPanel.setOpaque(false);
 		zoomPanel.setLayout(new BoxLayout(zoomPanel, BoxLayout.X_AXIS));
 		JLabel zoomLabel = new JLabel("Zoom:   ");
 		zoomComboBox = new JComboBox();
@@ -423,6 +446,7 @@ public class StandaloneGUI extends UmletGUI {
 		zoomComboBox.setMaximumSize(zoomComboBox.getPreferredSize());
 		zoomListener = new ZoomListener();
 		zoomComboBox.addActionListener(zoomListener);
+		zoomComboBox.addMouseWheelListener(zoomListener);
 		zoomComboBox.setToolTipText("Use ± or Ctrl+mouse wheel to zoom");
 
 		String[] zoomValues = new String[] { "10%", "20%", "30%", "40%", "50%", "60%", "70%", "80%", "90%", "100%", "110%", "120%", "130%", "140%", "150%", "160%", "170%", "180%", "190%", "200%" };
@@ -435,7 +459,7 @@ public class StandaloneGUI extends UmletGUI {
 
 		menu.add(zoomPanel);
 
-		mailButton = new JButton("Mail diagram");
+		mailButton = new JToggleButton("Mail diagram");
 
 		mailButton.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
@@ -460,18 +484,30 @@ public class StandaloneGUI extends UmletGUI {
 		rightSplit.setDividerSize(2);
 		rightSplit.setDividerLocation(Constants.right_split_position);
 		rightSplit.setResizeWeight(1);
-		rightSplit.setBorder(null);
+		rightSplit.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0), 0));
+		rightSplit.setAlignmentX(Component.RIGHT_ALIGNMENT);
 
-		mainSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, diagramspanel, rightSplit);
+		paletteControlsPanel = new JPanel();
+		paletteControlsPanel.setLayout(new BoxLayout(paletteControlsPanel, BoxLayout.X_AXIS));
+		paletteControlsPanel.add(paletteList);
+
+		rightPanel = new JPanel();
+		rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
+		rightPanel.add(paletteControlsPanel);
+		rightPanel.add(rightSplit);
+		paletteList.setAlignmentX(Component.CENTER_ALIGNMENT);
+		rightSplit.setAlignmentX(Component.CENTER_ALIGNMENT);
+
+		mainSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, diagramspanel, rightPanel);
 		mainSplit.setDividerSize(2);
 		mainSplit.setDividerLocation(Constants.main_split_position);
 		mainSplit.setResizeWeight(1);
-		mainSplit.setBorder(null);
+		mainSplit.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0), 0));
 
 		customSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT, mainSplit, customPanel);
 		customSplit.setDividerSize(0);
 		customSplit.setResizeWeight(1);
-		customSplit.setBorder(null);
+		customSplit.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0), 0));
 		customPanel.setVisible(false);
 
 		mailPanel = this.createMailPanel();
@@ -479,14 +515,11 @@ public class StandaloneGUI extends UmletGUI {
 		mailSplit.setDividerSize(0);
 		mailSplit.setDividerLocation(Constants.mail_split_position);
 		mailSplit.setResizeWeight(1);
-		mailSplit.setBorder(null);
+		mailSplit.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0), 0));
 		mailPanel.setVisible(false);
 
-		DividerListener dividerListener = new DividerListener();
-		customSplit.addComponentListener(dividerListener);
-		rightSplit.addComponentListener(dividerListener);
-		mainSplit.addComponentListener(dividerListener);
-		mailSplit.addComponentListener(dividerListener);
+		// Adding the DividerListener which refreshes Scrollbars here is enough for all dividers
+		palettepanel.addComponentListener(new DividerListener());
 
 		/******************* ADD PALETTES ***************************/
 		Hashtable<String, PaletteHandler> palettetable = umlet.getPalettes();
@@ -526,24 +559,10 @@ public class StandaloneGUI extends UmletGUI {
 	public void selectPalette(String palette) {
 		if (this.selected_palette.equals(palette)) return;
 
-		for (Component c : this.menu_palettes.getMenuComponents()) {
-			if (c instanceof JMenuItem) {
-				JMenuItem item = (JMenuItem) c;
-				if (item.getText().equals(this.selected_palette)) {
-					// item.setEnabled(true);
-					item.setIcon(null);
-					item.repaint();
-				}
-				else if (item.getText().equals(palette)) {
-					// item.setEnabled(false);
-					item.setIcon(new ImageIcon("icons/ok.gif"));
-					item.repaint();
-				}
-			}
-		}
-
 		this.selected_palette = palette;
 
+		paletteList.setSelectedItem(this.selected_palette);
+
 		CardLayout cl = (CardLayout) (this.palettepanel.getLayout());
 		cl.show(this.palettepanel, this.getSelectedPalette());
 	}
@@ -581,7 +600,6 @@ public class StandaloneGUI extends UmletGUI {
 		if (count > 0) {
 			menu_edit_delete.setEnabled(true);
 			menu_edit_cut.setEnabled(true);
-			// menu_edit_copy.setEnabled(true);
 			if (count > 1) menu_edit_group.setEnabled(true);
 			else menu_edit_group.setEnabled(false);
 		}
@@ -595,8 +613,8 @@ public class StandaloneGUI extends UmletGUI {
 	}
 
 	@Override
-	public void enablePaste() {
-		menu_edit_paste.setEnabled(true);
+	public void setPaste(boolean value) {
+		menu_edit_paste.setEnabled(value);
 	}
 
 	@Override
@@ -606,37 +624,43 @@ public class StandaloneGUI extends UmletGUI {
 
 	@Override
 	public void setCustomPanelEnabled(boolean enable) {
+		log.info("Set CustomPanel enabled=" + Boolean.toString(enable));
 		this.custom_panel_visible = enable;
 		if (this.customPanel.isVisible() != enable) {
 			int loc = this.mainSplit.getDividerLocation();
+			log.info("mainSplit Divider Location: " + loc);
+
+			this.customPanel.setVisible(enable);
+
 			if (enable) {
 				int rightloc = this.rightSplit.getDividerLocation();
-				this.customSplit.setDividerLocation(rightloc);
+				log.info("rightSplit Divider Location: " + rightloc);
 				this.customSplit.setDividerSize(2);
-				this.mainSplit.setRightComponent(this.palettepanel);
-				this.customPanel.getRightSplit().setDividerLocation(loc);
-				this.customPanel.getRightSplit().updateUI();
+				this.rightSplit.setDividerSize(0);
+
 				this.customPanel.getLeftSplit().setLeftComponent(this.propertyTextPane.getPanel());
+				log.info("customPanel.leftSplit Divider Location: " + customPanel.getLeftSplit().getDividerLocation());
+
+				this.customSplit.setDividerLocation(rightloc);
+				this.customPanel.getRightSplit().setDividerLocation(loc);
 				this.customPanel.getLeftSplit().setDividerLocation(this.diagramspanel.getWidth() / 2);
+				log.info("customPanel.leftSplit New Divider Location: " + customPanel.getLeftSplit().getDividerLocation());
 				this.customPanel.getLeftSplit().updateUI();
 			}
 			else {
 				int rightloc = this.customSplit.getDividerLocation();
+				log.info("customSplit Divider Location: " + rightloc);
 				this.customSplit.setDividerSize(0);
-				this.mainSplit.setRightComponent(this.rightSplit);
-				this.rightSplit.setLeftComponent(this.palettepanel);
+
+				this.rightSplit.setDividerSize(2);
 				this.rightSplit.setRightComponent(this.propertyTextPane.getPanel());
 				this.rightSplit.setDividerLocation(rightloc);
-				this.rightSplit.updateUI();
-				this.customPanel.getLeftSplit().setDividerLocation(this.diagramspanel.getWidth() / 2);
 			}
-
 			this.mainSplit.setDividerLocation(loc);
+			log.info("mainSplit Divider Location: " + loc);
 			this.menu_custom_edit.setEnabled(!enable && this.custom_element_selected);
 			this.menu_custom_new.setEnabled(!enable);
 			this.menu_custom_template.setEnabled(!enable);
-			this.customPanel.setVisible(enable);
-
 		}
 		this.setDiagramsEnabled(!enable);
 	}
@@ -645,11 +669,11 @@ public class StandaloneGUI extends UmletGUI {
 	public void setMailPanelEnabled(boolean enable) {
 		if (enable) {
 			mailSplit.setDividerSize(2);
-			mailButton.setBackground(Constants.GRID_COLOR);
+			mailButton.setSelected(true);
 		}
 		if (!enable) {
 			mailSplit.setDividerSize(0);
-			mailButton.setBackground(null);
+			mailButton.setSelected(false);
 		}
 		mailPanel.setVisible(enable);
 		if (enable) mailSplit.setDividerLocation(Constants.mail_split_position);
@@ -669,33 +693,64 @@ public class StandaloneGUI extends UmletGUI {
 
 	@Override
 	public void diagramSelected(DiagramHandler handler) {
-		if (handler == null) {
-			menu_file_save.setEnabled(false);
-			menu_file_save_as.setEnabled(false);
-			menu_file_export_as_bmp.setEnabled(false);
-			menu_file_export_as_eps.setEnabled(false);
-			menu_file_export_as_gif.setEnabled(false);
-			menu_file_export_as_jpg.setEnabled(false);
-			menu_file_export_as_pdf.setEnabled(false);
-			menu_file_export_as_png.setEnabled(false);
-			menu_file_export_as_svg.setEnabled(false);
-			menu_edit.setEnabled(false);
+		updateGreyedOutMenuItems(handler);
+	}
+
+	public void updateGreyedOutMenuItems(DiagramHandler handler) {
+
+		// These menuitems only get changed if this is not the palette or custompreview
+		if (!(handler instanceof PaletteHandler) && !(handler instanceof CustomPreviewHandler)) {
+			if ((handler == null) || handler.getDrawPanel().getAllEntities().isEmpty()) {
+				menu_file_save.setEnabled(false);
+				menu_file_save_as.setEnabled(false);
+				menu_file_export_as.setEnabled(false);
+				menu_file_export_as_bmp.setEnabled(false);
+				menu_file_export_as_eps.setEnabled(false);
+				menu_file_export_as_gif.setEnabled(false);
+				menu_file_export_as_jpg.setEnabled(false);
+				menu_file_export_as_pdf.setEnabled(false);
+				menu_file_export_as_png.setEnabled(false);
+				menu_file_export_as_svg.setEnabled(false);
+				menu_file_mail_to.setEnabled(false);
+				menu_file_print.setEnabled(false);
+				mailButton.setEnabled(false);
+			}
+			else {
+				menu_file_save.setEnabled(true);
+				menu_file_save_as.setEnabled(true);
+				menu_file_export_as.setEnabled(true);
+				menu_file_export_as_bmp.setEnabled(true);
+				menu_file_export_as_eps.setEnabled(true);
+				menu_file_export_as_gif.setEnabled(true);
+				menu_file_export_as_jpg.setEnabled(true);
+				menu_file_export_as_pdf.setEnabled(true);
+				menu_file_export_as_png.setEnabled(true);
+				menu_file_export_as_svg.setEnabled(true);
+				menu_file_mail_to.setEnabled(true);
+				menu_file_print.setEnabled(true);
+				mailButton.setEnabled(true);
+			}
+		}
+
+		// The menu_edit menuitems always work with the actual selected diagram (diagram, palette or custompreview), therefore we change it everytime
+		if ((handler == null) || handler.getDrawPanel().getAllEntities().isEmpty()) {
+			menu_edit_copy.setEnabled(false);
+			menu_edit_select_all.setEnabled(false);
 		}
 		else if (handler instanceof CustomPreviewHandler) {
 			menu_edit.setEnabled(false); // do not enable delete / cut commands
 		}
 		else {
-			menu_file_save.setEnabled(true);
-			menu_file_save_as.setEnabled(true);
-			menu_file_export_as_bmp.setEnabled(true);
-			menu_file_export_as_eps.setEnabled(true);
-			menu_file_export_as_gif.setEnabled(true);
-			menu_file_export_as_jpg.setEnabled(true);
-			menu_file_export_as_pdf.setEnabled(true);
-			menu_file_export_as_png.setEnabled(true);
-			menu_file_export_as_svg.setEnabled(true);
-			menu_edit.setEnabled(true);
+			menu_edit.setEnabled(true); // must be set to enabled explicitely because it could be deactivated from CustomPreview
+			menu_edit_copy.setEnabled(true);
+			menu_edit_select_all.setEnabled(true);
 		}
+
+		if ((handler == null) || !handler.getController().isUndoable()) menu_edit_undo.setEnabled(false);
+		else menu_edit_undo.setEnabled(true);
+		if ((handler == null) || !handler.getController().isRedoable()) menu_edit_redo.setEnabled(false);
+		else menu_edit_redo.setEnabled(true);
+
 	}
 
 	@Override
@@ -712,7 +767,6 @@ public class StandaloneGUI extends UmletGUI {
 			c.setEnabled(enable);
 		for (int i = 0; i < this.diagramtabs.getTabCount(); i++)
 			this.diagramtabs.getTabComponentAt(i).setEnabled(enable);
-		this.menu_palettes.setEnabled(enable);
 		this.menu_file.setEnabled(enable);
 		this.searchField.setEnabled(enable);
 	}
@@ -752,6 +806,7 @@ public class StandaloneGUI extends UmletGUI {
 	public void setPropertyPanelText(String text) {
 		if (this.propertyTextPane != null) { // needed because of convert function
 			this.propertyTextPane.setText(text);
+			this.propertyTextPane.checkPanelForSpecialChars();
 			// Reset the vertical and horizontal scrollbar position to the upper left corner
 			this.propertyTextPane.setCaretPosition(0);
 		}
@@ -773,10 +828,6 @@ public class StandaloneGUI extends UmletGUI {
 		dialog.setVisible(true);
 	}
 
-	public String getValueOfZoomDisplay() {
-		return (String) zoomComboBox.getSelectedItem();
-	}
-
 	public void setValueOfZoomDisplay(int i) {
 		// This method should just set the value without ActionEvent therefore we remove the listener temporarily
 		if (zoomComboBox != null) {
