@@ -5,7 +5,9 @@ import java.awt.event.MouseListener;
 
 import javax.swing.JTabbedPane;
 
+import com.umlet.control.Umlet;
 import com.umlet.control.diagram.DiagramHandler;
+import com.umlet.gui.base.UmletGUI;
 
 public class TabListener implements MouseListener {
 
@@ -32,6 +34,12 @@ public class TabListener implements MouseListener {
 	public void mousePressed(MouseEvent arg0) {
 		this.pane.setSelectedComponent(handler.getDrawPanel().getScrollPanel());
 		this.handler.getDrawPanel().getSelector().updateSelectorInformation();
+
+		// If this is the standalone client we must also set the zoom value on the upper panel
+		UmletGUI gui = Umlet.getInstance().getGUI();
+		if ((gui != null) && (gui instanceof StandaloneGUI)) {
+			((StandaloneGUI) Umlet.getInstance().getGUI()).setValueOfZoomDisplay(handler.getGridSize());
+		}
 	}
 
 	public void mouseReleased(MouseEvent arg0) {
