@@ -4,8 +4,11 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Hashtable;
 import java.util.List;
+import java.util.Timer;
+import java.util.TimerTask;
 import java.util.Vector;
 
+import org.apache.log4j.Logger;
 import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.action.IMenuManager;
@@ -44,12 +47,24 @@ public class UMLetContributor extends EditorActionBarContributor {
 		COPY, CUT, PASTE, SELECTALL
 	}
 
+	private static Logger log = Logger.getLogger(UMLetContributor.class);
+
+	private static boolean diagramActionLocked = false;
+	private static final int DELAY = 100;
+
+	private class DelayTimerTask extends TimerTask {
+		@Override
+		public void run() {
+			diagramActionLocked = false;
+		}
+	}
+
 	private Action help;
 	private Action samples;
 	private Action website;
 	private Action rateUmlet;
 	private Action about;
-	private ArrayList<Action> zoomactions;
+	private ArrayList<ZoomAction> zoomactions;
 	private ArrayList<Action> exportactions;
 	private Action customnew;
 	private List<String> newfromtemplate;
@@ -58,6 +73,7 @@ public class UMLetContributor extends EditorActionBarContributor {
 	private Action customtutorial;
 	private Action options;
 	private Action sendPerMail;
+	private Action editPalette;
 
 	private IAction undoActionGlobal;
 	private IAction redoActionGlobal;
@@ -82,7 +98,7 @@ public class UMLetContributor extends EditorActionBarContributor {
 
 	public UMLetContributor() {
 		this.newfromtemplate = new ArrayList<String>();
-		this.zoomactions = new ArrayList<Action>();
+		this.zoomactions = new ArrayList<ZoomAction>();
 		this.exportactions = new ArrayList<Action>();
 		this.templateactions = new Hashtable<String, Action>();
 		createMenuActions();
@@ -129,8 +145,13 @@ public class UMLetContributor extends EditorActionBarContributor {
 		copyActionDiagram = new Action() {
 			@Override
 			public void run() {
-				DiagramHandler handler = Umlet.getInstance().getDiagramHandler();
-				if (handler != null) (new Copy()).execute(handler);
+				// After the call all diagram actions get locked for a short time period to avoid a bug with zoom to 100% before the command
+				if (!diagramActionLocked) {
+					diagramActionLocked = true;
+					DiagramHandler handler = Umlet.getInstance().getDiagramHandler();
+					if (handler != null) handler.getController().executeCommand(new Copy());
+					new Timer().schedule(new DelayTimerTask(), DELAY);
+				}
 			}
 		};
 		copyActionDiagram.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_COPY));
@@ -138,8 +159,13 @@ public class UMLetContributor extends EditorActionBarContributor {
 		cutActionDiagram = new Action() {
 			@Override
 			public void run() {
-				DiagramHandler handler = Umlet.getInstance().getDiagramHandler();
-				if (handler != null) handler.getController().executeCommand(new Cut());
+				// After the call all diagram actions get locked for a short time period to avoid a bug with zoom to 100% before the command
+				if (!diagramActionLocked) {
+					diagramActionLocked = true;
+					DiagramHandler handler = Umlet.getInstance().getDiagramHandler();
+					if (handler != null) handler.getController().executeCommand(new Cut());
+					new Timer().schedule(new DelayTimerTask(), DELAY);
+				}
 			}
 		};
 		cutActionDiagram.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_CUT));
@@ -148,8 +174,13 @@ public class UMLetContributor extends EditorActionBarContributor {
 		pasteActionDiagram = new Action() {
 			@Override
 			public void run() {
-				DiagramHandler handler = Umlet.getInstance().getDiagramHandler();
-				if (handler != null) handler.getController().executeCommand(new Paste());
+				// After the call all diagram actions get locked for a short time period to avoid a bug with zoom to 100% before the command
+				if (!diagramActionLocked) {
+					diagramActionLocked = true;
+					DiagramHandler handler = Umlet.getInstance().getDiagramHandler();
+					if (handler != null) handler.getController().executeCommand(new Paste());
+					new Timer().schedule(new DelayTimerTask(), DELAY);
+				}
 			}
 		};
 		pasteActionDiagram.setImageDescriptor(sharedImages.getImageDescriptor(ISharedImages.IMG_TOOL_PASTE));
@@ -250,7 +281,7 @@ public class UMLetContributor extends EditorActionBarContributor {
 		private String zoomFactor;
 
 		private ZoomAction(String name, String zoomFactor) {
-			super(name);
+			super(name, IAction.AS_RADIO_BUTTON);
 			this.zoomFactor = zoomFactor;
 		}
 
@@ -295,10 +326,10 @@ public class UMLetContributor extends EditorActionBarContributor {
 				BrowserLauncher.openURL("http://www.umlet.com/");
 			}
 		};
-		rateUmlet = new Action("Rate UMLet at EclipsePluginCentral...") {
+		rateUmlet = new Action("Rate UMLet at Eclipse Marketplace...") {
 			@Override
 			public void run() {
-				BrowserLauncher.openURL("http://www.eclipseplugincentral.com/modules.php?op=modload&name=Web_Links&file=index&req=ratelink&lid=465&ttitle=UMLet_-_UML_Tool_for_Fast_UML_Diagrams");
+				BrowserLauncher.openURL("http://marketplace.eclipse.org/content/umlet-uml-tool-fast-uml-diagrams");
 			}
 		};
 		about = new Action("About UMLet") {
@@ -410,6 +441,37 @@ public class UMLetContributor extends EditorActionBarContributor {
 				Umlet.getInstance().getGUI().setMailPanelEnabled(!Umlet.getInstance().getGUI().isMailPanelVisible());
 			}
 		};
+
+		this.editPalette = new Action("Edit Current Palette") {
+			@Override
+			public void run() {
+
+			// NYI because would only be operable in eclipse 3.3
+			// see eclipsepedia faq for more information.
+			/*
+			 * File fileToOpen = new File("externalfile.xml");
+			 * if (fileToOpen.exists() && fileToOpen.isFile()) {
+			 * IFileStore fileStore = EFS.getLocalFileSystem().getStore(fileToOpen.toURI());
+			 * IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
+			 * try {
+			 * IDE.openEditor...
+			 * } catch ( PartInitException e ) {
+			 * }
+			 * }
+			 */
+
+			// String filePath = Umlet.getInstance().getPalette().getFileHandler().getFullPathName();
+			// IFile file = workspaceRoot.getFileForLocation(path);
+			// IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
+			// try {
+			// IDE.openEditor(page, input);
+			// } catch ( PartInitException e ) {
+			// }
+
+			// http://wiki.eclipse.org/FAQ_How_do_I_open_an_editor_on_a_file_in_the_workspace%3F
+			// http://wiki.eclipse.org/FAQ_How_do_I_open_an_editor_on_a_file_outside_the_workspace%3F
+			}
+		};
 	}
 
 	@Override
@@ -424,7 +486,7 @@ public class UMLetContributor extends EditorActionBarContributor {
 		IMenuManager help = new MenuManager("Help");
 		manager.prependToGroup(IWorkbenchActionConstants.MB_ADDITIONS, menu);
 
-		for (Action a : this.zoomactions)
+		for (ZoomAction a : this.zoomactions)
 			zoom.add(a);
 
 		for (Action a : this.exportactions)
@@ -450,14 +512,16 @@ public class UMLetContributor extends EditorActionBarContributor {
 
 		menu.add(zoom);
 		menu.add(export);
-		menu.add(sendPerMail);
 		menu.add(custom);
+		// menu.add(editPalette);
+		menu.add(sendPerMail);
+		menu.add(new Separator());
 		menu.add(help);
 		menu.add(options);
 	}
 
-	public void enablePaste() {
-		this.pasteActionDiagram.setEnabled(true);
+	public void setPaste(boolean value) {
+		this.pasteActionDiagram.setEnabled(value);
 	}
 
 	public void setCustomElementSelected(boolean selected) {
@@ -520,4 +584,15 @@ public class UMLetContributor extends EditorActionBarContributor {
 
 		org.eclipse.swt.widgets.Display.getDefault().asyncExec(new UpdateActionBars(this.getActionBars()));
 	}
+
+	public void setZoomLevel(float zoomFactor) {
+		zoomFactor = zoomFactor * 10;
+		Integer factorAsInteger = (int) zoomFactor;
+		String factorAsString = factorAsInteger.toString();
+		for (ZoomAction a : this.zoomactions) {
+			if (a.zoomFactor.equals(factorAsString)) a.setChecked(true);
+			else a.setChecked(false);
+		}
+
+	}
 }
